function [Output_PAR, varargout] = logit(userPrefs, Initial_Parameters, Connections, Nesting_Structure, Types, varargin)
% logit (userPrefs, Initial_Parameters, Connections, Nesting_Structure, Types, varargin)
%
% INPUTS:
%
% USERPREFS (String or Structure) If a text string, the file output will be named with the string.
%	If a structure, activate the following fields to control the listed features.
%	All fields are optional. Default values in (paren).
%	.Model				Choose from:  (MNL)  NL  MXL  
%	.FileName			A text string to use as the filename for log and report files. 
%	.ReportTitle		A text string to use as the title for report files.
%						This is useful when multiple reports are generated in one file
%	.NumDraws			Number of draws to used in simulation for Mixed Logit (100)
%	.DrawStyle			The method to use for making simulation draws
%						Choose from:  MLH  halton  (scramble)  shuffle
%	.Scaling			Whether to scale variables, which can improve direction finding (true)/false
%	.ReuseDraws			Whether to re-use simulation draws, if the conditions are identical (true)/false
%	.VerboseOutput		Display the complete set of parameters at every iteration true/(false)
%	.SuppressAllOutput	Skip all output during processing, only give a final report at completion true/(false)
%	.Truncate_Size 		Truncation for unbounded normal distribution draws (10)
%	.Signif_Digits		Number of significant digits to report (3)
%	.Epsilon			Threshold for tolerance factor to stop optimization (1e-4)
%	.Test				Skip optimization, run a single LL iteration true/(false)
%	.OptimToolbox		Override the custom optimization algorithm, and ue the Matlab
%						Optimization Toolbox, if availalbe true/(false)
%	.Robust				Use robust sandwich estimator for covariance (true)/false
%	.HessDirection		Use the Hessian or approximate Hessian, instead of the BHHH,
%						to find the descent direction. (true)/false
%
% INITIAL_PARAMETERS (Vector or Structure of Vectors) Initial parameters vector(s), and constrained
%	values will not deviate from these values. If a vector is specified, that [row] vector
%	is the inital values for the BETA parameters, in the order they appear in the varargin. 
%	If a structure is spefified, a row vector can be specified in each of these fields:
%	.BETA	The MNL or NL Beta parameters, or the location parameters for MXL
%	.THETA	The spread parameters for MXL
%	.MU		The logsum parameters for NL
%
% CONNECTIONS (Array or Structure of Arrays) An array of connections. Use the same layout as
%	the INITIAL_PARAMETERS array. Enter different values in different slots to constrain the
%	parameters to those ratios. For multiple independent relationship connections, use additional
%	rows. Each row-set of connections applies across structure fields.
%
% NESTING_STRUCTURE (Vector) A vector that uniquely describes the nesting structure.
%	Begin with each alternative 1...N and number that slot in the vector with the address of the parent nest.
%	Parent nests are meta-alternatives, numbers starting one greater than the highest alternative.
%	Final, top level nest gets a pointer to itself.
%
% TYPES (String or Structure of Strings) If only one string is specified, it is the parameter types.
%	If a structure, types can be specified in these fields:
%	.Parameter	Parameter types (see below)
%	.LogSum		Constrain or unconstrain only
%	.Variable	Advanced Feature for accomodating new data formats.
%	
% 	PARAMETER_TYPES
%	One character per variable for each type, characters in order as variables appear at the end:
%	(P) Unconstrained point (not mixed) 
%	(p) Constrained point (not mixed) 
%	(N) Normal, Unconstrained  
%	(n) Normal, Constrained	 
%	(M) Normal, with unconstrained location but constrained spread
%	(m) Normal, with constrained location but unconstrained spread
%	(U) Uniform, Unconstrained	
%	(u) Uniform, Constrained  
%	(W) Uniform, with unconstrained location but constrained spread	 
%	(w) Uniform, with constrained location but unconstrained spread	 
%	(G) Gumbel, Unconstrained  
%	(g) Gumbel, Constrained	 
%	(H) Gumbel, with unconstrained location but constrained spread	
%	(h) Gumbel, with constrained location but unconstrained spread	
%	(T) Triangle, Unconstrained
%	(t) Triangle, Constrained
%	(R) Triangle, with unconstrained location but constrained spread
%	(r) Triangle, with constrained location but unconstrained spread
%	(L) Log-Normal, Unconstrained  
%	(l) Log-Normal, Constrained	 
%	(O) Log-Normal, with unconstrained location but constrained spread
%	(o) Log-Normal, with constrained location but unconstrained spread
%
% VARARGIN (Col Vectors) Any number of column vectors containing core data and/or explanitory variables.
%	Unless Variable_Types are specified, the data must start with CASE, ALT, and CHOSEN, and then
%	consist of variables in CaseID/AltID format only.
%
% This m-file written by Jeffrey Newman. Last update July 26 2005.
% Do not modify this file without changing this header information.

global Version_Number
Version_Number = 46;
clear global Keys
clear global Num

% GLOBALS:
global Time_Begin 	% Time		The time that the logit estimation process begins
global ITERATION	% Scalar	The iteration number, used for reporting interim results
global LogFile		% Ref #		Developed when the log file is opened/created
global ReportFile	% Ref #		Developed when the report file is opened/created
global LogCheck		% Scalar	Current Byte location in the Log File, used to confirm write commands
global Var_Names	% Structure	The names of variables, used in reporting results
					% .BETA		The names of the variables
					% .THETA	The names of spread parameters
					% .MU		The names of the logsum parameters
global Num 			% Structure	Numerical information about the data and models
					% .Vars		These should be self-explanitory
					% .Cases
					% .Alts
					% .CaseAlts
					% .K_CaseAlt	= Num.Cases * Num.Alts
					% .Nests
					% .LogSums
					% .Random_Vars
					% .NonRandom_Vars
					% .PAR.BETA
					% .PAR.THETA
					% .PAR.MU
					% .PAR.TOTAL
					% .Draws
					%
global Prefs		% Structure	The preferences to use in running the code
					%			See user help (above) for details
global Module		% Structure	Various switches to activate/deactivate parts of the code
					%			based on the attibutes of the models and data
					% .MNL				Run the MNL model
					% .Nested			Run the NL model
					% .MXL				Run the MXL model
					% .Connections		Identifies whether any connections between parameters
					%					are defined
					% .BFGS				Use the BFGS approximation of the hessian instead of an
					%					analytic hessian (NL and MXL models)
					% .RepeatedChoice   Identifies whether repeated choices are present
					% .BinaryChoice		Identifies whether choices are strictly yes or no
					%					as opposed to fraction choices from stated preference
					% .Panel			Identifies panel data (used in MXL models)
					% .ConfirmData		Whether to pause after preparing data to allow user
					%					to inspect the data set-up and make sure it is correct
					% .Only_X			Identifies if all the data is CaseID/AltID format only
					% .DrawStyle.MLH 		Use modified latin hypercube draws
					% .DrawStyle.Halton 	Use halton draws			
					% .DrawStyle.Scramble	Use scrambled halton draws
					% .DrawStyle.Shuffle	Use shuffled halton draws
global DATA			% Structure	The observed variables, created by logit_data.m
					% .Z	Availability
					% .Y	Chosen alternatives
					% .X	The CaseID-AltID variables
					% .W	Case weights (R-type variables only)
					% .R	Case-identical variables
					% .M	Class varying case-identical variables
					% .O	Classes in CaseIDonly format
					% .G	Panel/Group in CaseID-AltID format
					% .Y_NL	Chosen alternatives and nests
					% .Max_Adj.A	The adjustment factors
					% .Max_Adj.L	The adjustment factors for Log Normal distributed parameters
					% .Max_Adj.N	The adjustment factors for other distributed parameteres
global Keys 		% Structure of information about constrained, connected, and unconstrained
					% .Free		 Vector indicating the unconstrained variables
					% .Lock		 Vector indicating the constrained or connected variables
					% .Unlock	 Vector indicating the correct order of the parameters
					% .Connector Matrix of connections for linear algebra
global Current_Parameters % The current iteration parameters, recorded for restartability

Num_Fixed_Arg = 5;	% The number of inputs to this function preceding the varargin
ITERATION = 0;		% Initializes iteration counter


Module.ConfirmData = false;									% Default setting to not wait for confirmation
logit_prefs(userPrefs);										% Set a priori preferences for logit.m
ReportFile = fopen(['Report ' Prefs.FileName '.txt'],'a+');	% Define and open log and report files
LogFile = fopen(['Log of ' Prefs.FileName '.txt'],'a+');	% Open a log file in the current directory to record the ongoing process.
LogCheck = ftell(LogFile);									% Check the current byte of the log file (to ensure results are recording).
Time_Begin = clock;											% Get the time this run begins.
timestamp('Begin Time');									% Record the begin time in the log file.
fprintf('\nThis program is logit.m version %0.0f.\n',Version_Number)
for n = 1:length(varargin)									% Get variable names
	Variable_Names{n} = inputname(Num_Fixed_Arg + n);		%
end		%

[Parameter_Types] = logit_data(Types, Variable_Names, varargin{:});	% Create DATA structure

if isfield(Types, 'LogSum')									% Create LogSum_Types if they are not defined
	LogSum_Types = Types.LogSum;							%
else														%
	LogSum_Types = 'P';										%
end															%

[BETA, THETA, MU, Keys, Simple_Structure] = logit_parameterize(Initial_Parameters, Connections, Nesting_Structure, Parameter_Types, LogSum_Types);

% Check for unused user preferences
if isstruct(userPrefs)
	Fields = fieldnames(userPrefs);
	for i = 1:length(Fields)
		warn_check(isfield(Prefs,Fields{i}),'WARNING: %s is not a valid logit.m preference\n',Fields{i})
	end % i
end % isstruct



%%%%% Confirm Data Structure %%%%%

% Confirmation report of variable names and order
relay('Data Structure Results:\n');

Var_Names.THETA = [];
Var_Names.THETA(1,:) = [Var_Names.BETA(1,:), ' Spread'];

Var_InitialReport = strvcat(Var_Names.BETA, 'Variables ');

report     (['\n========================================================================\n']);
report('%-10s ', Var_InitialReport(end,:));
if Module.MXL
	report ('Initial Beta  Initial Spread  '); 
else
	report ('InitialParam  ');
end
report		(  'Distribution  Constraint\n')

for EachVar = 1:size(Var_Names.BETA,1)
	report ('%-10s ', Var_Names.BETA(EachVar,:));

	report ('%-14.3g',BETA(EachVar)); 
	if Module.MXL
		report ('%-16.3g',THETA(EachVar)); 
	end
	
	switch Parameter_Types(EachVar)
	case {'P'}
		report ('Point         unconstrained\n');				
	case {'p'}
		report ('Point         constrained\n');		
	case {'N'}
		report ('Normal        unconstrained\n');				
	case {'n'}
		report ('Normal        constrained\n');		
	case {'M'}
		report ('Normal        unconstrained location, constrained spread\n');		
	case {'m'}
		report ('Normal        constrained location, unconstrained spread\n');		
	case {'G'}
		report ('Gumbel        unconstrained\n');		
	case {'g'}
		report ('Gumbel        Constrained\n');		
	case {'H'}
		report ('Gumbel        unconstrained location, constrained spread\n');		
	case {'h'}
		report ('Gumbel        constrained location, unconstrained spread\n');		
	case {'U'}
		report ('Uniform       unconstrained\n');		
	case {'u'}
		report ('Uniform       Constrained\n');		
	case {'W'}
		report ('Uniform       unconstrained location, constrained spread\n');		
	case {'w'}
		report ('Uniform       constrained location, unconstrained spread\n');		
	case {'T'}
		report ('Triangle      unconstrained\n');		
	case {'t'}
		report ('Triangle      Constrained\n');		
	case {'R'}
		report ('Triangle      unconstrained location, constrained spread\n');		
	case {'r'}
		report ('Triangle      constrained location, unconstrained spread\n');		
	case {'L'}
		report ('Log-Normal    unconstrained\n');		
	case {'l'}
		report ('Log-Normal    Constrained\n');		
	case {'O'}
		report ('Log-Normal    unconstrained location, constrained spread\n');		
	case {'o'}
		report ('Log-Normal    constrained location, unconstrained spread\n');		
	end
	Var_Names.THETA(EachVar,:) = [Var_Names.BETA(EachVar,:) ' Spread'];	
end % EachVar
for EachMu = 1:Num.LogSums*Module.Nested
	relay ('%-10s ', Var_Names.MU(EachMu,:));
	report ('%-14.3g',MU(EachMu)); 
	switch LogSum_Types(EachMu)
	case {'P'}
		report ('Point         unconstrained\n');				
	case {'p'}
		report ('Point         constrained\n');		
	end
end

% if Module.ConfirmData, pause; end



timestamp;	%End of Data Preparations
	
	
%%%%% MODELS %%%%%
	
if Module.MNL, 	  [BETA] = run_MNL(BETA); end
if Module.Nested, [BETA, MU] = run_NL (BETA, MU, Simple_Structure); end									
if Module.MXL, 	  [BETA, THETA] = run_MXL(BETA, THETA, Parameter_Types); end



% Record processing time on report
Total_Seconds = etime(clock,Time_Begin);
Total_Minutes = fix(Total_Seconds/60);
Total_Seconds = rem(Total_Seconds,60);
report ('\nTotal Time for Model Estimation: %2.0f minutes', Total_Minutes);
report (' %1.2f seconds\n========================================================================.\n', Total_Seconds);



% Fix Parameter_Table into two-row format for final return

Output_PAR.BETA = BETA;
if ~isempty(THETA)
	Output_PAR.THETA = THETA;
end
if ~isempty(MU)
	Output_PAR.MU = MU;
end

% Housekeeping
timestamp('End Time');
fclose(LogFile);				% Closes the LogFile, allowing other applications to access it
fclose(ReportFile);				% Closes the ReportFile, allowing other applications to access it

fprintf ('Processing Complete - All Data Recorded to Disk\n');
beep;

%-----------------------------------------------------------------------
% This code is for a unix terminal, to assemble the logit.m library into one file.
% cat logit.m logit_prefs.m logit_data.m logit_parameterize.m  do_min.m min_in_this_direction.m run_MNL.m run_NL.m run_MXL.m LL_MNL_X.m LL_MNL_XRM.m LL_NL.m LL_MXL.m create_lock_unlock_PAR.m create_working_PAR.m create_simple_structure.m draw_hypercube.m halton.m trianglize.m gumbelize.m relay.m report.m write_report.m timestamp.m error_check.m warn_check.m negative_choleski.m truncate.m > logit45.m
%-----------------------------------------------------------------------
function [] = logit_prefs(userPrefs)

global Module Num Prefs

% Default Preferences
Prefs.Model				= 'MNL';		%  MNL  NL  MXL
Prefs.NumDraws			=  100;
Prefs.FileName			= 'logit';
Prefs.ReportTitle		= 'A Model';
Prefs.DrawStyle			= 'scramble';	%  MLH  halton  scramble  shuffle
Prefs.Scaling			= true;
Prefs.ReuseDraws		= true;
Prefs.VerboseOutput 	= false;
Prefs.SuppressAllOutput = false;
Prefs.Truncate_Size 	= 10;			% Truncation for unbounded normal distribution draws
Prefs.Signif_Digits 	= 3;			% Number of significant digits to report
Prefs.Epsilon			= 1e-4;			% Threshold for tolerance factor to stop optimization
Prefs.Test				= false;		% Skip optimization and just get LL for parameters
Prefs.OptimToolbox		= false;		% Override built-in optimization and use MATLAB standard
Prefs.Robust			= false;		% Use robust sandwich estimator for covariance matrix 
Prefs.X_NegDuo			= false;		% Experimental
Prefs.X_Mirror			= false;		% Experimental
Prefs.X_Population		= false;		% Experimental
Prefs.X_NegPanel		= false;		% Experimental

% Override defaults with user defined values
if ischar(userPrefs)
	Prefs.ReportTitle = userPrefs;
	Prefs.FileName = userPrefs;
end
Fields = fieldnames(Prefs);
for i = 1:length(Fields)
	if isfield(userPrefs, Fields{i})
		Prefs.(Fields{i}) = userPrefs.(Fields{i});
	end % isfield
end % i

% Apply preferences to relevant items
Num.Draws = Prefs.NumDraws;

% Select Model(s) to run
if     strcmp(Prefs.Model, 'MNL')
	Module.MNL = 	true;
	Module.Nested = false;
	Module.MXL = 	false;
elseif strcmp(Prefs.Model, 'NL')
	Module.MNL = 	false;
	Module.Nested = true;
	Module.MXL = 	false;
elseif strcmp(Prefs.Model, 'MXL')
	Module.MNL = 	false;
	Module.Nested = false;
	Module.MXL = 	true;
else
	Module.MNL = 	true;
	Module.Nested = false;
	Module.MXL = 	false;
end


% Select Draw type for MXL model
if     strcmp(Prefs.DrawStyle, 'MLH')
	Module.DrawStyle.MLH	 = 	true;			
	Module.DrawStyle.Halton = 	false;			
	Module.DrawStyle.Scramble = false;
	Module.DrawStyle.Shuffle = 	false;
elseif strcmp(Prefs.DrawStyle, 'halton')
	Module.DrawStyle.MLH	 = 	false;			
	Module.DrawStyle.Halton = 	true;			
	Module.DrawStyle.Scramble = false;
	Module.DrawStyle.Shuffle = 	false;
elseif strcmp(Prefs.DrawStyle, 'scramble')
	Module.DrawStyle.MLH	 = 	false;			
	Module.DrawStyle.Halton = 	false;			
	Module.DrawStyle.Scramble = true;
	Module.DrawStyle.Shuffle = 	false;
elseif strcmp(Prefs.DrawStyle, 'shuffle')
	Module.DrawStyle.MLH	 = 	false;			
	Module.DrawStyle.Halton = 	false;			
	Module.DrawStyle.Scramble = false;
	Module.DrawStyle.Shuffle = 	true;
else
	Module.DrawStyle.MLH	 = 	true;			
	Module.DrawStyle.Halton = 	false;			
	Module.DrawStyle.Scramble = false;
	Module.DrawStyle.Shuffle = 	false;
end

% Activate BFGS approximate hessian for MXL and NL models
if Module.MNL
	Module.BFGS = false;
else
	Module.BFGS = true;
end

%-----------------------------------------------------------------------
function [Parameter_Types] = logit_data(Types, Variable_Names, varargin)
% DATA Format
%	Type of Distribution
%				P	Unconstrained point (not mixed) 
%				p	Constrained point (not mixed) 
%				N	Normal, Unconstrained  
%				n	Normal, Constrained	 
%				M	Normal, with unconstrained location but constrained spread
%				m	Normal, with constrained location but unconstrained spread
%				U	Uniform, Unconstrained	
%				u	Uniform, Constrained  
%				W	Uniform, with unconstrained location but constrained spread	 
%				w	Uniform, with constrained location but unconstrained spread	 
%				G	Gumbel, Unconstrained  
%				g	Gumbel, Constrained	 
%				H	Gumbel, with unconstrained location but constrained spread	
%				h	Gumbel, with constrained location but unconstrained spread	
%				T	Triangle, Unconstrained
%				t	Triangle, Constrained
%				R	Triangle, with unconstrained location but constrained spread
%				r	Triangle, with constrained location but unconstrained spread
%				L	Log-Normal, Unconstrained  
%				l	Log-Normal, Constrained	 
%				O	Log-Normal, with unconstrained location but constrained spread
%				o	Log-Normal, with constrained location but unconstrained spread
%	Type of Variable
%			  Available Choices in ID Case/ID Alt Format
%				C	Case ID
%				A	Alt ID
%				G	Group or Panel ID
%				Y	Chosen Alternative
%				X	Regular Explanitory Variable
%				Z	Availability of Alternative (do not input, created later)
%				E	Segmentation variable
%			  Available Choices in ID Case Format
%				H	Group or Panel ID
%				W	Weight of Case
%				D	Chosen Alternative Number
%				S	Alternative-Specific Variable		NOT YET AVAILABLE
%				O	Origin (for Distance Matrix)
%			  Available Choices in ID Alt Format
%				R	Case-Identical Variable (all cases face same values for each alternative)
%				M	Distance Matrix (special Case-Identical Variable, varying by Origin)

clear global DATA
global DATA Var_Names Num Module Prefs

% Default Types
Orig_Parameter_Types = 'P';
Variable_Types = 'CAYX';
M_Types = 1;

if ischar(Types)
	Orig_Parameter_Types = Types;
end
if isfield(Types,'Parameter')
	Orig_Parameter_Types = Types.Parameter;
end
if isfield(Types,'Variable')
	Variable_Types = Types.Variable;
end
if isfield(Types,'M')
	M_Types = Types.M;
end



% Extend Variable_Types, repeat last one until full.
while length(Variable_Types) < length(varargin)
	Variable_Types(end+1) = Variable_Types(end);
end

% Identify the various Variable_Type locations
C = regexp(Variable_Types,'[C]');
A = regexp(Variable_Types,'[A]');
G = regexp(Variable_Types,'[G]');
H = regexp(Variable_Types,'[H]');
Y = regexp(Variable_Types,'[Y]');
X = regexp(Variable_Types,'[X]');
R = regexp(Variable_Types,'[R]');
D = regexp(Variable_Types,'[D]');
O = regexp(Variable_Types,'[O]');
W = regexp(Variable_Types,'[W]');
M = regexp(Variable_Types,'[M]');
E = regexp(Variable_Types,'[E]');

Case_Only = regexp(Variable_Types,'[WDSO]');
Alts_Only = regexp(Variable_Types,'[RM]');
Variables_Only = [X R M];


% Extend Parameter_Types, repeat last one until full.
while length(Orig_Parameter_Types) < length(Variables_Only)
	Orig_Parameter_Types(end+1) = Orig_Parameter_Types(end);
	Module.ConfirmData = true;
end
for v = 1:length(Variables_Only)
	Edit_Parameter_Types(Variables_Only(v)) = Orig_Parameter_Types(v);
end

% Extend M_Types, repeat last one until full.
while length(M_Types) < length(M)
	M_Types(end+1) = M_Types(end);
end

% Initialize variables
DATA.X = [];
Var_Names.BETA = [];
Var_Names.THETA = [];
Parameter_Types = [];

if ~isempty(C)
	% Define global core variables using Case-Alt Data					
	relay ('Defining global core variables using Case-Alt Data...\n');
	
	CASES = varargin{C};
	ALTS = varargin{A};
	
	[Unique_CASES, Translate_CASES, CASES] = unique([CASES]);	% Ensure consecutive whole numbering for cases
	[Unique_ALTS , Translate_ALTS , ALTS] = unique([ALTS]);	% Ensure consecutive whole numbering for alternatives

	Num.Cases = length(Unique_CASES);
	Num.Alts = length(Unique_ALTS);
	Num.CaseAlts = length([CASES]);			% includes only available alternatives
	Num.K_CaseAlt = Num.Cases*Num.Alts;		% includes all alternatives in each case
	
	Availability = logical(0);
	Availability(Num.Cases,Num.Alts) = 0;
	Chosen = zeros(Num.Cases,Num.Alts);
	
	QuickData = [varargin{X}];	% Load data into a matrix for speed
	X_CAV = zeros(Num.Cases,Num.Alts,length(X));
	
	relay ('Building Data Array for Case-Alt Data...\n');
	for EachCaseAlt = 1:Num.CaseAlts						% Fills these three arrays from the flat data files
		Availability(CASES(EachCaseAlt), ALTS(EachCaseAlt)) = 1;
		Chosen      (CASES(EachCaseAlt), ALTS(EachCaseAlt)) = varargin{Y}(EachCaseAlt);
		for V = 1:length(X)
			X_CAV(CASES(EachCaseAlt), ALTS(EachCaseAlt), V) = QuickData(EachCaseAlt,V); % varargin{X(V)}(EachCaseAlt); % 
		end % V
	end % EachCaseAlt in Build Matrices
	X_CAV = reshape(X_CAV,[Num.K_CaseAlt,length(X)]);
	DATA.X = X_CAV;
	for x = X
		Var_Names.BETA = strvcat(Var_Names.BETA, Variable_Names{x});
		Var_Names.THETA = strvcat(Var_Names.THETA, [Variable_Names{x} ' Spread']);
	end % r
	Parameter_Types = [Parameter_Types Edit_Parameter_Types(X)];

else
	% Define global core variables using Case-Alt Data					
	relay ('Defining global core variables using Case-Only Data...\n');
	
	for EachH = (1:length(H)) + length(G)
		GRP(EachH).Names = varargin{H(EachH)};
		[Unique_GROUPS, Translate_GROUPS, GRP(EachH).Names] = unique(GRP(EachH).Names);	% Ensure consecutive whole numbering for groups
		GRP(EachH).Num = length(Unique_GROUPS);
	end	
	
	Num.Cases = length(varargin{Case_Only(1)});
	Num.Alts = size(varargin{Alts_Only(1)},1);
	Num.K_CaseAlt = Num.Cases*Num.Alts;		
	Num.CaseAlts = Num.K_CaseAlt;		
		
end

Num.Vars = length(regexp(Variable_Types,'[XRM]'));

relay ('Writing Data Structure...\n');


% Chosen Alternatives
if ~isempty(Y)
	DATA.Y = Chosen;
elseif ~isempty(D)
	DATA.Y = sparse(1:Num.Cases,varargin{D},true,Num.Cases,Num.Alts);
end

% Check for Repeated Choice
if max(max(DATA.Y)) > 1
	Module.RepeatedChoice = true;
else
	Module.RepeatedChoice = false;
end

% Check for Binary Choice
if all(all(DATA.Y == (DATA.Y~=false)))	% Check if all values are binary
	Module.BinaryChoice = true;			% If so, set Module.BinaryChoice to 'on', will improve speed
else
	Module.BinaryChoice = false;		% When all choices are either 1 or 0, calculation speed can be enhanced
end

% Available Alternatives
if ~isempty(C)
	DATA.Z = Availability;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% For Regular CaseID/AltID data, the remaining data construction is not operative %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Case-Only Format Variables
DATA.R = [];
INFO.ParamTypes.R = [];
if ~isempty(R)
	DATA.R = [varargin{R}];
	for r = R
		Var_Names.BETA = strvcat(Var_Names.BETA, Variable_Names{r});
		Var_Names.THETA = strvcat(Var_Names.THETA, [Variable_Names{r} ' Spread']);
	end % r
	Parameter_Types = [Parameter_Types Edit_Parameter_Types(R)];
end

% Distance Matrix Format Variables
DATA.M = cell(length(O),1);
INFO.ParamTypes.M = [];
if ~isempty(M)
	for EachM = 1:length(M)
		m = M(EachM);
		DATA.M{M_Types(EachM)} = [DATA.M{M_Types(EachM)} reshape(varargin{m},[size(varargin{m},1) 1 size(varargin{m},2)])];
		Var_Names.BETA = strvcat(Var_Names.BETA, Variable_Names{m});
		Var_Names.THETA = strvcat(Var_Names.THETA, [Variable_Names{m} ' Spread']);
	end % r
	Parameter_Types = [Parameter_Types Edit_Parameter_Types(M)];
end

% Origins
DATA.O = [];
if ~isempty(O)
	DATA.O = [varargin{O}];
end

% Case Weights
DATA.W = [];
if ~isempty(W)
	DATA.W = varargin{W};
end


% Group Finding for social module or panel data only
if ~isempty(G)									
	relay ('Finding Groups...\n');
	DATA.G.Names = varargin{G};
	[Unique_GROUPS, Translate_GROUPS, DATA.G.Names] = unique(DATA.G.Names);	% Ensure consecutive whole numbering for groups
	DATA.G.Num = length(Unique_GROUPS);
	DATA.G.Cases = zeros(Num.Cases, 1);
	DATA.G.Duos = false(1,1,Num.Cases);
	DATA.G.FlatBegins(1) = 1;											% The first group begins with the first case.											
	for EachGroup = 1:DATA.G.Num										% For each group,
		for EachCaseAlt = DATA.G.FlatBegins(EachGroup):Num.CaseAlts		% from the beginning of this group to the end of the data
			if (EachCaseAlt == Num.CaseAlts)							% if the end of the data is found,
				DATA.G.FlatEnds(EachGroup) = EachCaseAlt;				% this group ends at the end of the data,
				break;													% and there is no need to continue the search.
			elseif (DATA.G.Names(EachCaseAlt) ~= DATA.G.Names(EachCaseAlt+1))		% Otherwise, if next CaseAlt is in a new group,
				DATA.G.FlatEnds(EachGroup) = EachCaseAlt; 				% then this group ends with this CaseAlt
				DATA.G.FlatBegins(EachGroup+1) = EachCaseAlt + 1;		% and the next group begins with the next CaseAlt
				break;													% and there is no need to continue the search for this group end.
			end % if
		end % search for Current_Group_End
		DATA.G.Begins(EachGroup) = CASES(DATA.G.FlatBegins(EachGroup));	% definition of Group Begins for array data
		DATA.G.Ends  (EachGroup) = CASES(DATA.G.FlatEnds  (EachGroup));	% definition of Group Ends for array data
		DATA.G.Cases(DATA.G.Begins(EachGroup):DATA.G.Ends(EachGroup)) = EachGroup;
		DATA.G.NumCases(EachGroup) = DATA.G.Ends(EachGroup) - DATA.G.Begins(EachGroup) +1;
		DATA.G.Duos(DATA.G.Ends(EachGroup)) = true;
		DATA.G.DuosK = reshape(repmat(DATA.G.Duos,[1 1 1 Num.Alts]),[Num.K_CaseAlt,1]);
	end % EachGroup
	Module.Panel = true;
else
	DATA.G.Begins = 1:Num.Cases;
	DATA.G.Ends = DATA.G.Begins;
	DATA.G.Num = Num.Cases;
	DATA.G.Cases = 1:Num.Cases;
	DATA.G.NumCases = ones(size(DATA.G.Cases));
	DATA.G.Duos = 1;
	DATA.G.DuosK = 1;
	Module.Panel = false;
end % ~isempty(G)


% Record Revised Data Ordering
DATA.RevisedOrder = [X R M];

% Set Module for CaseID/AltID data only
if length(X) == Num.Vars
	Module.Only_X = true;
else	
	Module.Only_X = false;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Data Scaling for Possible Improved Perfomance %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DATA.Max_Adj.N = ones(1,Num.Vars);	% Define adjustment for non-lognormal variables			
DATA.Max_Adj.L = ones(1,Num.Vars);	% Define adjustment for lognormal variables			
DATA.Max_Adj.A = ones(1,Num.Vars);	% Define adjustment for all variables			

if Prefs.Scaling == 1
	relay ('Scaling variables...\n');

	for x = 1:size(DATA.X,2)												% For each variable in the data,
		switch Parameter_Types(x)
		case {'N','n','M','m','G','g','H','h','U','u','W','w','T','t','R','r','P','p'}
			DATA.Max_Adj.N(x) = max(max(abs(DATA.X(:,x))));			% find the largest magnitude value,
			DATA.X(:,x) = DATA.X(:,x) / DATA.Max_Adj.N(x);	% Divides that variable in all cases by the largest value
			DATA.Max_Adj.A(x) 		  = DATA.Max_Adj.N(x);
		case {'L','l','O','o'}
			% Scaleing for Lognormally distributed parameters is more complex
			DATA.Max_Adj.L(x) = max(max(abs(DATA.X(:,x))));			% find the largest magnitude value,
			DATA.X(:,x) = DATA.X(:,x) / DATA.Max_Adj.L(x);	% Divides that variable in all cases by the largest value
			DATA.Max_Adj.A(x)		  = DATA.Max_Adj.L(x);
		end
	end
	
	x = size(DATA.X,2);
	
	for r = 1:size(DATA.R,2)												% For each variable in the data,
		switch Parameter_Types(r+x)
		case {'N','n','M','m','G','g','H','h','U','u','W','w','T','t','R','r','P','p'}
			DATA.Max_Adj.N(r+x) = max(max(abs(DATA.R(:,r))));			% find the largest magnitude value,
			DATA.R(:,r) = DATA.R(:,r) / DATA.Max_Adj.N(r+x);	% Divides that variable in all cases by the largest value
			DATA.Max_Adj.A(r+x) 		  = DATA.Max_Adj.N(r+x);
		case {'L','l','O','o'}
			% Scaleing for Lognormally distributed parameters is more complex
			DATA.Max_Adj.L(r+x) = max(max(abs(DATA.R(:,r))));			% find the largest magnitude value,
			DATA.R(:,r) = DATA.R(:,r) / DATA.Max_Adj.L(r+x);	% Divides that variable in all cases by the largest value
			DATA.Max_Adj.A(r+x)		  = DATA.Max_Adj.L(r+x);
		end
	end

	x = x + size(DATA.R,2);

	for o = 1:length(DATA.M)
		for m = 1:size(DATA.M{o},2)												% For each variable in the data,
			switch Parameter_Types(m+x)
			case {'N','n','M','m','G','g','H','h','U','u','W','w','T','t','R','r','P','p'}
				DATA.Max_Adj.N(m+x) = max(max(abs(DATA.M{o}(:,m,:))));			% find the largest magnitude value,
				DATA.M{o}(:,m,:) = DATA.M{o}(:,m,:) / DATA.Max_Adj.N(m+x);	% Divides that variable in all cases by the largest value
				DATA.Max_Adj.A(m+x) 		  = DATA.Max_Adj.N(m+x);
			case {'L','l','O','o'}
				% Scaleing for Lognormally distributed parameters is more complex
				DATA.Max_Adj.L(m+x) = max(max(abs(DATA.M{o}(:,m,:))));			% find the largest magnitude value,
				DATA.M{o}(:,m,:) = DATA.M{o}(:,m,:) / DATA.Max_Adj.L(m+x);	% Divides that variable in all cases by the largest value
				DATA.Max_Adj.A(m+x)		  = DATA.Max_Adj.L(m+x);
			end
		end % m
		x = x + size(DATA.M{o},2);
	end % o
	
end


%-----------------------------------------------------------------------
function [BETA, THETA, MU, Keys, Simple_Structure] = logit_parameterize(Initial_Parameters, Connections, Nesting_Structure, Parameter_Types, LogSum_Types)

global Module Num DATA Var_Names Prefs

% Reformat Nesting_Structure into usable matrix forms 
Num.Nests = length(Nesting_Structure);
Num.LogSums = max(Num.Nests - Num.Alts - 1,0);
%if Module.Nested
	[Simple_Structure, Dense_Structure] = create_simple_structure(Nesting_Structure);
	DATA.Y_NL = DATA.Y * Dense_Structure;
%end

% Extend LogSum_Types, repeat last one until full.
while length(LogSum_Types) < Num.LogSums
	LogSum_Types(end+1) = LogSum_Types(end);
end

% Count possible parameters
Num.PAR.BETA 	= Num.Vars;
Num.PAR.THETA 	= Num.Vars * Module.MXL;
Num.PAR.MU 		= Num.LogSums * Module.Nested;
Num.PAR.TOTAL 	= Num.PAR.BETA + Num.PAR.THETA + Num.PAR.MU;

Var_Names.MU = [];											% Add names for LogSum Parameters in NL models
for EachNest = 1:Num.Nests-Num.Alts-1										% For each Nest Parameter,
	Var_Names.MU = strvcat(Var_Names.MU, ['Nest' num2str(EachNest)]);		% Add a variable name for that parameter
end

%%%%% Create initial parameters if they are not provided %%%%

if isstruct(Initial_Parameters)
	BETA = zeros(1,Num.PAR.BETA);
	try
		[C] = min(length(Initial_Parameters.BETA),Num.PAR.BETA);
		BETA(1,1:C) = Initial_Parameters.BETA;
	end
	THETA = ones(1,Num.PAR.THETA)/100;
	try
		[C] = min(length(Initial_Parameters.THETA),Num.PAR.THETA);
		THETA(1,1:C) = Initial_Parameters.THETA;
	end
	MU = ones(1,Num.PAR.MU);
	try
		[C] = min(length(Initial_Parameters.MU),Num.PAR.MU);
		MU(1,1:C) = Initial_Parameters.MU;
	end
else
	BETA = zeros(1,Num.PAR.BETA);
	THETA = ones(1,Num.PAR.THETA)/100;
	MU = ones(1,Num.PAR.MU);
	[C] = min(length(Initial_Parameters),Num.Vars);
	if C > 0
		BETA(1,1:C) = Initial_Parameters;
	end
end % isstruct



%%%%% Define which parameters are flexible, and which are constrained %%%%%

% Pre-define logical arrays of the appropriate size
Flex_PAR.BETA  = false(1, Num.PAR.BETA );	
Flex_PAR.THETA = false(1, Num.PAR.THETA);
Flex_PAR.MU    = false(1, Num.PAR.MU   );

% Initial designation of flexible parameters based on type
for V = 1:Num.Vars
	switch Parameter_Types(V)				% Examine the parameter type for this variable
	case 'P'									% For unconstrained single point parameters,
		Flex_PAR.BETA(V) = 1;					% 	Set the location param as flexible, leave the spread param constrained.
	case {'N','U','G','T','L','E'} 				% For unconstrained random parameters,
		Flex_PAR.BETA(V) = 1;					% 	Set both Location and spread parameters as flexible.
		Flex_PAR.THETA(V) = 1;
	case {'M','W','H','R','O','V'} 				% For unconstrained location / constrained spread random parameters
		Flex_PAR.BETA(V) = 1;					% 	Set the location param as flexible, leave the spread param constrained
	case {'m','w','h','r','o','v','Y','Z'} 		% For constrained location / unconstrained spread random parameters
		Flex_PAR.THETA(V) = 1;					% 	Set the spread param as flexible, leave the location param constrained
	end % Parameter_Types
end % V	
for V = 1:Num.LogSums
	switch LogSum_Types(V)				% Examine the parameter type for this variable
	case 'P'									% For unconstrained single point parameters,
		Flex_PAR.MU(V) = 1;						% 	Set the location param as flexible, leave the spread param constrained.
	end 	
end

%%%%% Use Connections to further limit flexible parameters %%%%%

% Create Keys.Connector, a full matrix of connections
Keys.Connector = eye(Num.PAR.TOTAL);

if isempty(Connections)
	Module.Connections = false;
else
	Module.Connections = true;
end

if Module.Connections

	% If Connections is a structure, combine the parts
	All_Connections = zeros(1,Num.PAR.TOTAL);
	if isstruct(Connections)
		if isfield(Connections,'BETA')
			[R,C] = size(Connections.BETA);
			All_Connections(1:R,1:C) = Connections.BETA;
		end
		if isfield(Connections,'THETA')
			[R,C] = size(Connections.THETA);
			All_Connections(1:R,(1:C)+Num.PAR.BETA) = Connections.THETA;
		end
		if isfield(Connections,'MU')
			[R,C] = size(Connections.MU);
			All_Connections(1:R,(1:C)+Num.PAR.BETA+Num.PAR.THETA) = Connections.MU;
		end
	else
		[R,C] = size(Connections);
		All_Connections(1:R,1:C) = Connections;
	end % isstruct
		
	% Scale Connections for variable scaling
	[R,C] = size(All_Connections);
	All_Connections(:,1:Num.PAR.BETA) = All_Connections(:,1:Num.PAR.BETA) .* repmat(DATA.Max_Adj.A,[R 1]);
	if Module.MXL
		All_Connections((1:Num.PAR.THETA)+Num.PAR.BETA) = All_Connections((1:Num.PAR.THETA)+Num.PAR.BETA) .* repmat(DATA.Max_Adj.A,[R 1]);
	end

	% Scale Connections internally, compile into Keys.Connector, and report progress
	for EachR = 1:R
		report('These Relational Constraints Are Imposed: \n');
		Row_Factors = find(All_Connections(EachR,:));
		Row_Leader = Row_Factors(1);
		Keys.Connector(Row_Leader,:) = All_Connections(EachR,:) / All_Connections(EachR,Row_Leader);
		if Row_Leader <= Num.PAR.BETA
			report('%s\t', Var_Names.BETA(Row_Leader,:));
			report('%3.4f\n', All_Connections(EachR,Row_Leader) / DATA.Max_Adj.A(Row_Leader) );
			
		elseif Row_Leader <= Num.PAR.BETA+Num.PAR.THETA
			report('%s\t', Var_Names.THETA(Row_Leader-Num.PAR.BETA,:));
			report('%3.4f\n', All_Connections(EachR,Row_Leader) / DATA.Max_Adj.A(Row_Leader-Num.PAR.BETA) );
			
		else
			report('%s\t', Var_Names.MU(Row_Leader-Num.PAR.BETA-Num.PAR.THETA,:));
			report('%3.4f\n', All_Connections(EachR,Row_Leader));
		end

		for EachRF = Row_Factors(2:end)
			Keys.Connector(EachRF,:) = 0; % All_Connections(EachR,:) / All_Connections(EachR,EachRF);
			
			if EachRF <= Num.PAR.BETA
				report('%s\t', Var_Names.BETA(EachRF,:));
				report('%3.4f\n', All_Connections(EachR,EachRF) / DATA.Max_Adj.A(EachRF) );
				Flex_PAR.BETA(EachRF) = 0;
				
			elseif EachRF <= Num.PAR.BETA+Num.PAR.THETA
				report('%s\t', Var_Names.THETA(EachRF-Num.PAR.BETA,:));
				report('%3.4f\n', All_Connections(EachR,EachRF) / DATA.Max_Adj.A(EachRF-Num.PAR.BETA) );
				Flex_PAR.THETA(EachRF-Num.PAR.BETA) = 0;
				
			else
				report('%s\t', Var_Names.MU(EachRF-Num.PAR.BETA-Num.PAR.THETA,:));
				report('%3.4f\n', All_Connections(EachR,EachRF));
				Flex_PAR.MU(EachRF-Num.PAR.BETA-Num.PAR.THETA) = 0;
			end
		end % EachRF
	end % EachR
end % Module Connections

if ~Module.MXL
	Flex_PAR.THETA(:) = []; 
end
if ~Module.Nested
	Flex_PAR.MU(:) = []; 
end

% Write the Keys to transform the parameters
Keys.Free = [Flex_PAR.BETA Flex_PAR.THETA Flex_PAR.MU];
Keys.Lock = ~Keys.Free;
Keys.Unlock = zeros(1, Num.PAR.TOTAL);										% Initializes unlock Key to the correct size
Keys.Unlock(~Keys.Lock) =   1:sum(~Keys.Lock);								% Numbers the unlocked spots in the plain Key in order
Keys.Unlock( Keys.Lock) = ( 1:sum( Keys.Lock) ) + sum(~Keys.Lock);			% Numbers the locked spots in the plain Key

%-----------------------------------------------------------------------
function [Best_LL, BETA, GRAD, iHESS, BHHH, Tolerance, Outer_Iteration] = do_min (TargetFunc, BETA, Epsilon, varargin)
% This is a customized optimization algorithm for the logit.m program.
% Thanks to Paul Ruud and Ken Train, whose GAUSS code for mixed logit models provided a template for this code.

global Module Prefs
global Status_Text

Tolerance = 1000000;
Outer_Iteration = 0;
Record_Step_Sizes = ones(10,1);
Try_Again = 0;
Best_Step = 0;
SwitchTimer = 0;

Status_Text = 'Computing Gradient';
[Best_LL, GRAD, HESS, BHHH] = feval (TargetFunc, BETA, varargin{:});
if ~Prefs.VerboseOutput
	relay('Log Likelihood: %0.7f\n', -Best_LL);
end % ~Prefs.VerboseOutput
if isempty(HESS)
	HESS = BHHH;
end
error_check ((det(HESS) ~= 0), 'Singular Hessian or BHHH, Cannot Continue, Try Different Initial Parameters');	
iHESS = inv(HESS);

while (Tolerance > Epsilon) | (Tolerance < 0)
	Outer_Iteration = Outer_Iteration + 1;
	
	BETA_old = BETA;
	iHESS_old = iHESS;
	GRAD_old = GRAD;
	
	relay('\n==================================\nGradient Calculation Checkpoint %2.0f\n',Outer_Iteration);
	
	Direction_HESS = (iHESS * -GRAD);
	Direction_BHHH = (inv(BHHH) * -GRAD);
	Direction_GRAD = -GRAD;
	
	Tolerance = Direction_HESS' * -GRAD;
	relay('Tolerance: %0.7f\n',Tolerance);
	relay('----------------------------------\n');
	
	if (Tolerance >= 0) & (Tolerance < Epsilon)
		break;
	elseif Tolerance < 0
		Direction_HESS = -Direction_HESS;
	end
	
	if isnan(Tolerance)
		Skip_Hessian = true;
	else
		Skip_Hessian = false;
	end % if
	
	Some_Improvement = false;
	
	%%% Search %%%
	if ~Skip_Hessian
		Status_Text = 'Testing Steps (Hessian Ascent)';
		Try_Step = mean(Record_Step_Sizes(end-1:end));
		[Best_LL, BETA, Some_Improvement, Best_Step, GRAD, iHESS, BHHH] = min_in_this_direction ...
									(TargetFunc, Direction_HESS, BETA, Best_LL, Try_Step, varargin{:});
	end
	
	if ~Some_Improvement
		relay('*** No improvement found using Hessian search method, trying BHHH method ***\n');
		Status_Text = 'Testing Steps (BHHH Ascent)';
		Try_Step = Try_Step * (2^-15);
		[Best_LL, BETA, Some_Improvement, Best_Step, GRAD, iHESS, BHHH] = min_in_this_direction ...
								(TargetFunc, Direction_BHHH, BETA, Best_LL, Try_Step, varargin{:});
		 % Discard old hessian information, it is invalid if the algorithm drops into BHHH
		 iHESS_old = inv(BHHH);
	end
	
	if ~Some_Improvement
		relay('*** No improvement found using Hessian search method, trying BHHH method ***\n');
		Status_Text = 'Testing Steps (Steepest Ascent)';
		Try_Step = Try_Step * (2^-15);
		[Best_LL, BETA, Some_Improvement, Best_Step, GRAD, iHESS, BHHH] = min_in_this_direction ...
								(TargetFunc, Direction_GRAD, BETA, Best_LL, Try_Step, varargin{:});
		 % Discard old hessian information, it is invalid if the algorithm drops into BHHH
		 iHESS_old = inv(BHHH);
	end
	
	if ~Some_Improvement
		relay ('*** No improvement found using Steepest Ascent, terminating search ***\n');
		report('* * * * * FAILED TO CONVERGE, THESE RESULTS MAY NOT BE VALID * * * * *\n');
		timestamp;
		break;
	end % if
	
	timestamp;
	
	if Best_Step ~= 0
		Record_Step_Sizes(end+1) = Best_Step;	% Record the step size
	end % if
	
	% Calculation of approximate Hessian using BFGS algorithm
	if Module.BFGS
		dBETA = BETA - BETA_old;
		dGRAD = GRAD - GRAD_old;
		Denom = dGRAD' * dBETA;
		part = dBETA - iHESS_old * dGRAD; 
		iHESS = iHESS_old + ( part * dBETA' + dBETA * part' ) / Denom - (dGRAD' * part) * dBETA * dBETA' / Denom^2;
	end % Module.BFGS
		
end % while Tolerance
relay('\n==============================================================\n\n');
if Tolerance < Epsilon
	relay('Converged with tolerance %0.7f\n',Tolerance);
else
	relay('Stopped with tolerance %0.7f\n',Tolerance);
end % if


%-----------------------------------------------------------------------
function [Best_Val, BETA_Best, Some_Improvement, Best_Step, GRAD, iHESS, BHHH] = min_in_this_direction ...
		 (func, Direction, BETA, Best_Val, Start_Step, varargin)
% This function complete a line search seeking to optimize func while moving along Direction
% It returns a gradient and hessian (analytic or approximate) at the minimum

Step = Start_Step;					% Initial step length is usually 1 for BHHH, smaller for steepest ascent
Tolerance = 0.00001 * Start_Step;	% Tolerance to quit continually improving
Some_Improvement = false;			% Marker to remember if any improvement is made
Best_Step = 0;						% Initial best step length found is zero
GRAD = zeros(size(BETA));			% Default GRADient
iHESS = zeros(length(BETA));		% Default HESSian
BHHH = [];							% Default BHHH

% Initial func evaluation at Step 1
[New_Val, GRAD1, HESS1, BHHH1] = feval(func, BETA + (Step * Direction), varargin{:}, Best_Val);
Num_Iteration = 1;
relay('\nStep %0.0f Value: %0.7g\n', Num_Iteration, Step);
relay('Log Likelihood: %0.7f\n', -New_Val);
relay('LL Improvement: %0.7g\n', Best_Val - New_Val);

% Depending on the result of the initial evalution, search above or below one
if New_Val < Best_Val
	Second_Val  = Best_Val;		% The start value is now second
	Second_Step = 0;			% The start step is now second
	Best_Val    = New_Val;		% The new best value 
	Best_Step   = Step;   		% The new best step
	GRAD		= GRAD1;		 
	iHESS		= inv(HESS1);	
	BHHH		= BHHH1;
	Some_Improvement = true;		
	Still_Improving = true;
	
	% This loop continues to extend the search in Direction as long as improvements are found
	while Still_Improving & (Num_Iteration < 3)
		Step = Step * 2;
		[New_Val, GRAD1, HESS1, BHHH1] = feval(func, BETA + (Step * Direction), varargin{:}, Best_Val);
		Num_Iteration = Num_Iteration + 1;
		relay('\nStep %0.0f Value: %0.7g\n', Num_Iteration, Step);
		relay('Log Likelihood: %0.7f\n', -New_Val);
		relay('LL Improvement: %0.7g\n', Best_Val - New_Val);
		if New_Val < Best_Val
			Second_Val  = Best_Val;		% The old best value is now second
			Second_Step = Best_Step;	% The old best step is now second
			Best_Val    = New_Val;		% The new best value 
			Best_Step   = Step;   		% The new best step
			GRAD		= GRAD1;		 
			iHESS		= inv(HESS1);
			BHHH		= (BHHH1);
			Still_Improving = 1;
		elseif New_Val < Second_Val
			Second_Val  = New_Val;		% The new value is now second
			Second_Step = Step;			% The new step is now second
			Still_Improving = false;
		else
			Still_Improving = false;
		end % if
	end % while	
	
	% This loop continues the line search by subdividing the best interval
	while (Second_Val - Best_Val > Tolerance) & (Num_Iteration < 0)
		Step = (Best_Step + Second_Step) / 2;
		[New_Val, GRAD1, HESS1, BHHH1] = feval(func, BETA + (Step * Direction), varargin{:}, Best_Val);
		Num_Iteration = Num_Iteration + 1;
		relay('\nStep %0.0f Value: %0.7g\n', Num_Iteration, Step);
		relay('Log Likelihood: %0.7f\n', -New_Val);
		relay('LL Improvement: %0.7g\n', Best_Val - New_Val);
		if New_Val < Best_Val
			Second_Val  = Best_Val;		% The old best value is now second
			Second_Step = Best_Step;	% The old best step is now second
			Best_Val    = New_Val;		% The new best value 
			Best_Step   = Step;   		% The new best step
			GRAD		= GRAD1;		 
			iHESS		= inv(HESS1);
			BHHH		= (BHHH1);
		elseif New_Val < Second_Val
			Second_Val  = New_Val;		% The new value is now second
			Second_Step = Step;			% The new step is now second
		end % if
	end % while
	
else % when the initial step degraded the result
	% This loop shinks the steps to seek an improvement
	while ~Some_Improvement & (Num_Iteration < 15)
		Step = Step / 2;
		[New_Val, GRAD1, HESS1, BHHH1] = feval(func, BETA + (Step * Direction), varargin{:}, Best_Val);
		Num_Iteration = Num_Iteration + 1;
		relay('\nStep %0.0f Value: %0.7g\n', Num_Iteration, Step);
		relay('Log Likelihood: %0.7f\n', -New_Val);
		relay('LL Improvement: %0.7g\n', Best_Val - New_Val);
		if New_Val < Best_Val
			Best_Step = Step;
			Best_Val = New_Val;
			Some_Improvement = 1;
			GRAD		= GRAD1;		 
			iHESS		= inv(HESS1);
			BHHH		= (BHHH1);
		end % if
	end	% while
end

% Final value recording
if Best_Step == 0		% When algorithm abandons search unsuccessfully
	Best_Step = Step;	% Report the last (smallest) step tried
	BETA_Best = BETA;
	Best_Val = Best_Val;
else
	BETA_Best = BETA + (Best_Step * Direction);
end

%-----------------------------------------------------------------------
function [BETA] = run_MNL(BETA)
% run_MNL(BETA, THETA, MU)
%	This function executes the selected optimization code for standard
%	multinomial logit models.
%
% INPUT:  BETA, a Row Vector, the initial BETA parameters on the variables in order.
% OUTPUT: BETA, a Row Vector, the final BETA parameters on the variables in order.

global Keys DATA Prefs Module Num Var_Names

relay ('Beginning MNL Model Estimation.\n');

BETA = BETA .* DATA.Max_Adj.A;
[BETA_open, BETA_lock] = create_lock_unlock_PAR (Keys.Free, BETA);

%%%%% Calculation of the Log Likelihood for the model %%%%%

% Non-optimization for testing a set of parameters
if Prefs.Test == 1		
	LOG_LIKE_ZERO = NaN;
	if Module.Only_X
		[LL, GRAD, HESS, BHHH] = LL_MNL_X (BETA_open, BETA_lock);			
	else
		[LL, GRAD, HESS, BHHH] = LL_MNL_XRM (BETA_open, BETA_lock);			
	end
	iHESS = inv(HESS);

% For using Matlab's Built-in Optimization Toolbox
elseif Prefs.OptimToolbox	
	Options = optimset('TolX',1e-03,'TolFun',1e-05, ...
				'MaxFunEvals',1e+100,'GradObj', 'on', 'Hessian', 'on');
	if Module.Only_X
		LOG_LIKE_ZERO = LL_MNL_X(zeros(size(BETA_open)), BETA_lock);
		[BETA_open, LL, ef, ou, GRAD, HESS] = fminunc(@LL_MNL_X, BETA_open, Options, BETA_lock);
	else
		LOG_LIKE_ZERO = LL_MNL_XRM(zeros(size(BETA_open)), BETA_lock);
		[BETA_open, LL, ef, ou, GRAD, HESS] = fminunc(@LL_MNL_XRM, BETA_open, Options, BETA_lock);
	end
	if Prefs.Robust
		[LL, GRAD, HESS, BHHH] = LL_MNL (BETA_open, BETA_lock);			
	end
	iHESS = inv(HESS);

% For using the logit.m optimization algorithm (default)
else 
	relay('MNL Model');
	if Module.Only_X
		LOG_LIKE_ZERO = LL_MNL_X(zeros(size(BETA_open)), BETA_lock);
		[LL, BETA_open, GRAD, iHESS, BHHH, Tol, Iter] = do_min ...
							(@LL_MNL_X, BETA_open, Prefs.Epsilon, BETA_lock);
	else
		LOG_LIKE_ZERO = LL_MNL_XRM(zeros(size(BETA_open)), BETA_lock);
		[LL, BETA_open, GRAD, iHESS, BHHH, Tol, Iter] = do_min ...
							(@LL_MNL_XRM, BETA_open, Prefs.Epsilon, BETA_lock);
	end
end

BETA = create_working_PAR (BETA_open, BETA_lock, Keys.Unlock, Keys.Connector);
BETA = BETA ./ DATA.Max_Adj.A;

relay ('BETA has been optimized for the MNL model.\n');

% Calculate covariance matrix of the parameter estimates
covariance_matrix = zeros(length(BETA));
if Prefs.Robust
	covariance_matrix(1:length(BETA_open),1:length(BETA_open)) = iHESS * BHHH * iHESS;	
else
	covariance_matrix(1:length(BETA_open),1:length(BETA_open)) = iHESS;
end
covariance_matrix = covariance_matrix(Keys.Unlock,Keys.Unlock);
covariance_matrix = covariance_matrix ./ (DATA.Max_Adj.A' * DATA.Max_Adj.A);

% Calculate t-statistics against 0
Warning_Div_Zero = warning('off', 'MATLAB:divideByZero'); % Temporary Div Zero Supression	
stderr = sqrt(diag(covariance_matrix(1:Num.Vars,1:Num.Vars)));
t_stat = BETA' ./ stderr;
warning(Warning_Div_Zero); % Revert DivZero Warning to original state

% Report of Results
write_report(Prefs.FileName, LL, LOG_LIKE_ZERO, Var_Names.BETA, BETA, ...
				t_stat, stderr, covariance_matrix, Tol, Iter);

%-----------------------------------------------------------------------
function [BETA, MU] = run_NL(BETA, MU, Simple_Structure)

global Prefs Var_Names Module DATA Keys Num

relay ('Beginning Nested Logit Model Estimation.\n');

BETA = BETA .* DATA.Max_Adj.A;
[PAR_open, PAR_lock] = create_lock_unlock_PAR (Keys.Free, BETA, [], MU);

Warning_Div_Zero = warning('off', 'MATLAB:divideByZero');	% Temporary Div Zero Supression

% Calculation of the Log Likelihood for the model

% Non-optimization for testing a set of parameters
if Prefs.Test
	LOG_LIKE_ZERO = NaN;
	[LL, GRAD, HESS, BHHH] = LL_NL (PAR_open, PAR_lock, Simple_Structure);
	HESS = zeros(length(PAR_open));
		
% For using Matlab's Built-in Optimization Toolbox
elseif Prefs.OptimToolbox	
	Options = optimset('TolX',1e-05,'TolFun',1e-05,'MaxFunEvals',1e+100);
	[LL0] = LL_NL(ZERO_open, PAR_lock, Simple_Structure);
	[PAR_open, LL, exitflag, output, GRAD, HESS] = fminunc(@LL_NL, PAR_open, Options, PAR_lock, CHOSEN, Simple_Structure);	
	if all(eig(HESS)>0)
		relay('All eigenvalues are positive at stopping point.\n')
		relay('This estimation has converged at a local optimum.\n')
	else
		relay('Warning: Not all eigenvalues are positive at stopping point.\n')
		relay('This estimation has NOT converged at a local optimum.\n')
	end % if
	if Prefs.Robust
		[LLr, GRADr, HESSr, BHHH] = LL_NL (PAR_open, PAR_lock, Simple_Structure);			
	end
	iHESS = inv(HESS);

% For using the logit.m optimization algorithm (default)
else	
	[ZERO_open, ZERO_lock] = create_lock_unlock_PAR (Keys.Free, zeros(size(BETA)), [], ones(size(MU)));
	[LL0] = LL_NL(ZERO_open, PAR_lock, Simple_Structure);
	[LL, PAR_open, GRAD, iHESS, BHHH, Tol, Iter] = do_min (@LL_NL, PAR_open, Prefs.Epsilon, PAR_lock, Simple_Structure);		
end

[BETA, THETA, MU] = create_working_PAR (PAR_open, PAR_lock, Keys.Unlock, Keys.Connector);
BETA = BETA ./ DATA.Max_Adj.A;

relay ('BETA has been optimized for the NL model. \n');
	
% Calculate unscaled BETA values
NL_PARs = [BETA MU];
	
% Calculate covariance matrix of the parameter estiamtes
covariance_matrix = zeros(length([BETA MU]));
if Prefs.Robust
	covariance_matrix(1:length(PAR_open),1:length(PAR_open)) = iHESS * BHHH * iHESS;
else
	covariance_matrix(1:length(PAR_open),1:length(PAR_open)) = iHESS;
end
covariance_matrix = covariance_matrix(Keys.Unlock,Keys.Unlock);
covariance_matrix(1:Num.Vars,1:Num.Vars) = covariance_matrix(1:Num.Vars,1:Num.Vars) ./ (DATA.Max_Adj.A' * DATA.Max_Adj.A);

% Calculate standard error of the estimate
stderr = sqrt(diag(covariance_matrix));

% Calculate t-statistics against 0
t_stat = [BETA MU]' ./ stderr;

% Revert DivZero Warning to original state
warning(Warning_Div_Zero);	

% Report of Results
write_report(Prefs.FileName, LL, LL0, strvcat(Var_Names.BETA,Var_Names.MU), [BETA MU], t_stat, stderr, covariance_matrix, Tol, Iter);

%-----------------------------------------------------------------------
function [BETA, THETA] = run_MXL (BETA, THETA, Parameter_Types)

global DATA Num Module Prefs Keys Var_Names Std_Draws Dist

% Distill the parameter types to random and non-random parameters
Dist.Random_Vars = regexp(Parameter_Types,'[NUGTnugtMWHRmwhrLlOoEeVvYZ]');% Find the index locations of the random variable types
Dist.Random_Var_Types = Parameter_Types(Dist.Random_Vars);				% Assemble a types string containing only the random variables
Dist.NonRandom_Vars = regexp(Parameter_Types,'[Pp]');					% Find the index locations of the non-random (fixed point) variable types
Dist.LogNormal_Vars = regexp(Parameter_Types,'[LlOoEeVv]');				% Find the index locations of the LogNormal variable types
Dist.All_Types = Parameter_Types;
Num.Random_Vars = length(Dist.Random_Vars);
Num.NonRandom_Vars = length(Dist.NonRandom_Vars);

DATA.VAC = permute(reshape(DATA.X,[Num.Cases,Num.Alts,Num.Vars]),[3 2 1]);

if Module.Panel & ~Prefs.X_NegPanel
	Num_Draw_Items = DATA.G.Num;
else
	Num_Draw_Items = Num.Cases;
end % if

% Random Draws
Need_To_Make_Draws = 1;		% Start with the assumption that new draws are required

% Option to re-use existing draws instead of making new ones
if Prefs.ReuseDraws == 1		
	
	% Define the requested draw attributes
	New_Draw_Attributes.Types = Dist.Random_Vars;
	New_Draw_Attributes.Size  = [Num.Draws, Num.Random_Vars, Num_Draw_Items];
	New_Draw_Attributes.Style = Module.DrawStyle;
	
	% Get existing draw attributes, while avoiding an error if they don't exist
	try
		Std_Draws_Attributes = Std_Draws.Attributes;
	catch
		Std_Draws_Attributes = [];
	end
	
	% Check if saved draws are available, and if the attributes match
	if isequal(New_Draw_Attributes, Std_Draws_Attributes)
		Need_To_Make_Draws = 0;
		relay ('\rUsing saved draws instead of calculating new ones...\n');
	end % if	
end % if

% Make new uniform draws if necessary
if Need_To_Make_Draws == 1
	if Module.DrawStyle.Halton
		relay ('\rCalculating Halton Draw...\n');
		Draws = halton ('halton_regular', Num.Draws, Num.Random_Vars, Num_Draw_Items);
		relay ('Complete\n');		
	elseif Module.DrawStyle.Scramble
		relay ('\rCalculating Scrambled Halton Draw...\n');
		Draws = halton ('halton_scramble', Num.Draws, Num.Random_Vars, Num_Draw_Items);
		relay ('Complete\n');		
	elseif Module.DrawStyle.Shuffle
		relay ('\rCalculating Shuffled Halton Draw...\n');
		Draws = halton ('halton_shuffle', Num.Draws, Num.Random_Vars, Num_Draw_Items);
		relay ('Complete\n');		
	else	
		relay ('\rBeginning Random Hypercube Draw...');
		Draws = draw_hypercube (Num.Draws, Num.Random_Vars, Num_Draw_Items);
		relay ('Complete\n');
	end
	
	% Apply transformations to random variable draws to create standard draws
	% Draws are optionally truncated on infinite distributions to avoid massive magnitude values
	relay ('Computing Standard Draws...\n');
	Std_Draws.DVC = zeros(size(Draws));
	for EachRandomVar = 1:Num.Random_Vars;
		switch Dist.Random_Var_Types(EachRandomVar)	
		case {'N','n','M','m'}							% N,M: Normal
			Std_Draws.DVC(:,EachRandomVar,:) = truncate(norminv(Draws(:,EachRandomVar,:), 0, 1),Prefs.Truncate_Size);
		case {'L','l','O','o'}							% L,O: Log-Normal
			Std_Draws.DVC(:,EachRandomVar,:) = truncate(norminv(Draws(:,EachRandomVar,:), 0, 1),Prefs.Truncate_Size);
		case {'E','e','V','v'}							% E,V: Negative Log-Normal
			Std_Draws.DVC(:,EachRandomVar,:) = truncate(norminv(Draws(:,EachRandomVar,:), 0, 1),Prefs.Truncate_Size);
		case {'G','g','H','h'}							% G,H: Gumbel
			Std_Draws.DVC(:,EachRandomVar,:) = truncate(gumbelize(Draws(:,EachRandomVar,:), 0, 1),Prefs.Truncate_Size);
		case {'U','u','W','w'}							% U,W: Uniform
			Std_Draws.DVC(:,EachRandomVar,:) = (Draws(:,EachRandomVar,:) - 0.5) * 2;
		case {'T','t','R','r'}							% T,R: Triangle
			Std_Draws.DVC(:,EachRandomVar,:) = trianglize(Draws(:,EachRandomVar,:), 0, 1);
		case {'Y','Z'}									% N,M: Normal
			Std_Draws.DVC(:,EachRandomVar,:) = truncate(norminv(Draws(:,EachRandomVar,:), 0, 1),Prefs.Truncate_Size);
		end
	end % EachRandomVar
	
	if Module.Panel & ~Prefs.X_NegPanel
		Std_Draws.DVC = Std_Draws.DVC(:,:, DATA.G.Cases);	% Expands Draws with identical copies for each group member
	end % if
			
	if Prefs.X_NegDuo
		for D = 1:Num.Draws
		for V = 2:2:Num.Random_Vars		% Every alternate variable
			Std_Draws.DVC(D,V,:) = -Std_Draws.DVC(D,V-1,:);	% Mirror draws for the pos
			Std_Draws.DVC(D,V,DATA.G.Duos) = -Std_Draws.DVC(D,V,DATA.G.Duos);	% Mirror draws for the other person
		end
		end
	end % if
	
	if Prefs.X_Mirror
		Std_Draws.DVC_Mirror = zeros(size(Std_Draws.DVC));
		for D = 1:Num.Draws
		for V = 1:Num.Random_Vars		% Every alternate variable
			Std_Draws.DVC_Mirror(D,V,~DATA.G.Duos) = Std_Draws.DVC(D,V,~DATA.G.Duos);	% Copy draws for the first person
			Std_Draws.DVC_Mirror(D,V,DATA.G.Duos) = -Std_Draws.DVC(D,V,DATA.G.Duos);	% Mirror draws for the other person
		end
		end		
		Std_Draws.KRD_Mirror = repmat(permute(Std_Draws.DVC,[3 4 2 1]),[1 Num.Alts 1 1]);	% (C,A,V,D)	
		Std_Draws.KRD_Mirror = reshape(Std_Draws.KRD_Mirror, Num.Cases*Num.Alts, Num.Random_Vars, Num.Draws);		
	end

	if Prefs.X_NegPanel
		for EachGroup = 1:DATA.G.Num
			Group_Pop = DATA.G.NumCases(EachGroup);
			Choleski = negative_choleski(Group_Pop);
			for EachDraw = 1:Num.Draws
				Std_Draws.DVC(EachDraw,:,DATA.G.Begins(EachGroup):DATA.G.Ends(EachGroup)) = reshape(...
				reshape(Std_Draws.DVC(EachDraw,:,DATA.G.Begins(EachGroup):DATA.G.Ends(EachGroup)), Num.Random_Vars, Group_Pop) * Choleski, ...
				1, Num.Random_Vars, Group_Pop);
			end % EachDraw
		end % EachGroup
	end % Prefs.X_NegPanel

	% Reshape the Std_Draws values for gradient calculations
	Std_Draws.KRD = repmat(permute(Std_Draws.DVC,[3 4 2 1]),[1 Num.Alts 1 1]);	% (C,A,V,D)	
	Std_Draws.KRD = reshape(Std_Draws.KRD, Num.Cases*Num.Alts, Num.Random_Vars, Num.Draws);
	
	% Record the attributes of these draws for future comparison
	Std_Draws.Attributes.Types = Dist.Random_Vars;
	Std_Draws.Attributes.Size  = [Num.Draws, Num.Random_Vars, Num_Draw_Items];
	Std_Draws.Attributes.Style = Module.DrawStyle;		
			
end % if Need_To_Make_Draws


% Check for Binary Choice
if Module.BinaryChoice
	for EachDraw = 1:Num.Draws
		DATA.Y_D(:,:,EachDraw) = (DATA.Y~=0);
	end % EachDraw
else
	for EachDraw = 1:Num.Draws
		DATA.Y_D(:,:,EachDraw) = DATA.Y;
	end % EachDraw
end % if

timestamp; % Time to create draws

BETA = BETA .* DATA.Max_Adj.N;
BETA = BETA + log(DATA.Max_Adj.L);
THETA = THETA .* DATA.Max_Adj.N;

[PAR_open, PAR_lock] = create_lock_unlock_PAR (Keys.Free, BETA, THETA);

if Prefs.Test
	% One Pass of the function will find the LL, gradient, and BHHH
	Status_Text = 'Testing';
	[LL, GRAD, HESS, BHHH] = LL_MXL (PAR_open, PAR_lock, Std_Draws);
	iHESS = inv(HESS);
	LL0 = NaN;

% Regular Optimization using builtin MATLAB fminunc routine
elseif Prefs.OptimToolbox 
	Status_Text = 'Optimizing';
	[ZERO_open, ZERO_lock] = create_lock_unlock_PAR (Keys.Free, zeros(size(BETA)), zeros(size(THETA)));
	[LL0] = LL_MNL_X (ZERO_open, PAR_lock);
	Options = optimset('TolX',1e-03,'TolFun',1e-02,'MaxFunEvals',1e+100,'GradObj', 'on');
	[PAR_open, LL, exitflag, output, GRAD, HESS] = fminunc(@LL_MXL, PAR_open, Options, PAR_lock, Std_Draws);		
	if Prefs.Robust
		[LLr, GRADr, HESSr, BHHH] = LL_MXL (PAR_open, PAR_lock, Std_Draws);
	end
	iHESS = inv(HESS);
		
% For using the logit.m optimization algorithm (default)
else
	[ZERO_open, ZERO_lock] = create_lock_unlock_PAR (Keys.Free, zeros(size(BETA)), zeros(size(THETA)));
	[LL0] = LL_MNL_X (ZERO_open, PAR_lock);
	[LL, PAR_open, GRAD, iHESS, BHHH, Tol, Iter] = do_min (@LL_MXL, PAR_open, Prefs.Epsilon, PAR_lock, Std_Draws);
end % if Prefs.Test

% Results of the MXL Model

% Reconstruct the plain BETA and unscale
[BETA, THETA] = create_working_PAR (PAR_open, PAR_lock, Keys.Unlock, Keys.Connector);	
BETA = BETA - log(DATA.Max_Adj.L);
BETA = BETA ./ DATA.Max_Adj.N;
THETA = THETA ./ DATA.Max_Adj.N;

if Prefs.Robust
	iHESS = iHESS * BHHH * iHESS;
end

% Expand covariance matrices if necessary to full Parameter size
if length(PAR_lock) > 0
	iHESS(Num.Vars*2,Num.Vars*2) = 0;
end

% Re-sort rows & columns into plain ordering
iHESS = iHESS(Keys.Unlock,Keys.Unlock);

% Correct for scaling of parameters
iHESS = iHESS ./ ([DATA.Max_Adj.N DATA.Max_Adj.N]' * [DATA.Max_Adj.N DATA.Max_Adj.N]);

stderr = sqrt(diag(iHESS));		% Calculates standard errors

t_stat = [BETA THETA]' ./ stderr ;

% Calculate the mean and standard deviation of each coefficient
MXL_moments = zeros(Num.Vars,2);
for EachVar = 1:Num.Vars				% Consider Each Variable
	M = BETA(EachVar);										%	Location
	S = THETA(EachVar);										%	Spread
	switch Parameter_Types(EachVar)		% Consider the parameter type for each variable
	case {'P','p'}													% Fixed Point
		MXL_moments(EachVar,1) = M;									%	Only one value
		MXL_moments(EachVar,2) = 0;									%	Has no standard deviation
	case {'N','m','M','n'}											% Uniform
		MXL_moments(EachVar,1) = M;									%	Usual mean
		MXL_moments(EachVar,2) = S;									%	Usual standard deviation
	case {'L','l','O','o'}											% Log-Normal
		MXL_moments(EachVar,1) = exp(M+(S^2/2));					%	Mean
		MXL_moments(EachVar,2) = sqrt(exp(S^2+(2*M))*(exp(S^2)-1));	%	Standard Deviation
	case {'E','e','V','v'}											% Negative Log-Normal
		MXL_moments(EachVar,1) = -exp(M+(S/2));						%	Mean
		MXL_moments(EachVar,2) = sqrt(exp(S^2+(2*M))*(exp(S^2)-1));	%	Standard Deviation
	case {'U','u','W','w'}											% Uniform
		MXL_moments(EachVar,1) = M;									%	Usual mean
		MXL_moments(EachVar,2) = S / sqrt(3);	% 	Standard Deviation
	case {'T','t','R','r'}											% Triangle
		MXL_moments(EachVar,1) = M;									%	Usual mean
		MXL_moments(EachVar,2) = S / sqrt(6);	% 	Standard Deviation
	case {'G','g','H','h'}											% Gumbel
		MXL_moments(EachVar,1) = M + 0.577215665 * S;				%	Mean
		MXL_moments(EachVar,2) = pi * S / 6;						%	Standard Deviation
	end % switch					
end % EachVar

% Trim covariance matrix of spreads for points
trim_iHESS = iHESS;
isRand = true(Num.Vars*2,1);
for V = Num.Vars:-1:1
	switch Parameter_Types(V)		% Consider the parameter type for each variable
	case {'P','p'}													% Fixed Point
		trim_iHESS(Num.Vars+V,:) = [];
		trim_iHESS(:,Num.Vars+V) = [];
		isRand(Num.Vars+V) = false;
	end
end	

MXL_names = strvcat(Var_Names.BETA,Var_Names.THETA(isRand(Num.Vars+1:end),:));


% Report of Results
write_report(Prefs.FileName, LL, LL0, MXL_names, [BETA THETA(isRand(Num.Vars+1:end))],...
					t_stat(isRand), stderr(isRand), trim_iHESS, Tol, Iter);

% Supplemental Report with Moments
spacer = repmat(' ',[1 length(MXL_names(1,:))]);
spacer(1:9) = 'Variables';

Report_Format = ['%-9.' num2str(Prefs.Signif_Digits) 'g\t'];

report('Number of Draws: %0.0f\n',Prefs.NumDraws);
report('Draw Style: %s\n',Prefs.DrawStyle);

report([spacer '\tMean    \tStd Dev\n']);
for V = 1:Num.Vars
	report('%-9s\t', MXL_names(V,:));
	report(Report_Format,MXL_moments(V,1));
	report(Report_Format,MXL_moments(V,2));
	report('\n');		
end

%-----------------------------------------------------------------------
function [LL, GRAD, HESS, BHHH] = LL_MNL_X (PAR_open, PAR_lock, Best_LL)
% LL_MNL_X (PAR_open, PAR_lock, Best_LL)
% 	This function calculates the (negative of) log likelihood of a standard
%	multinomial logit model, given a set of parameters. It is optimized
%	for CaseID/AltID format data, (DATA.X) and will not function with any
%	other data format. The negative is returned to be compatible with
%	standard minimization algorithms, since the goal is to maximize LL.
%
% INPUT:
%	PAR_open	Col Vector	The unknown parameters, across which Log Likelihood is maximized
%	PAR_lock	Col Vector	Constrained parameters, across which Log Likelihood is maximized
%	Best_LL		Scalar		(optional) The best log likelihood found so far. This is used as
%							a trigger to skip the calculation of the gradient and hessian
%							if this iteration is not an improvement.
% OUTPUT:
%	LL			Scalar		The (negative of) log likelihood
%	GRAD		Col Vector	The vector of first order partial derivatives
%	HESS		Matrix		The matrix of second partial derivatives
%	BHHH		Matrix		The sum of the outer product of the gradient at each case
 
global LogFile LogCheck
global Prefs
global ITERATION 
global Var_Names 
global Current_Parameters
global Module
global DATA 
global Num 
global Connect 
global Keys 

ITERATION = ITERATION + 1;

BETA = create_working_PAR (PAR_open, PAR_lock, Keys.Unlock, Keys.Connector);

% Fast MNL model whel all of the variables are X-style

% Build Utility Matrices
Utility = DATA.X * (BETA');
Utility = reshape(Utility, Num.Cases, Num.Alts);
Utility = exp(Utility) .* DATA.Z;				% Set the exp_utility of unavailable alternatives to zero

% Build Cum_Utility and Probabilities Matrices
Cum_Utility = sum(Utility,2);					% Calculate total exp_utility in each case for the denominator
Probabilities = zeros(Num.Cases, Num.Alts);
for EachAlt = 1:Num.Alts
	Probabilities(:, EachAlt) = Utility(:, EachAlt) ./ Cum_Utility(:, 1);	% Find the logit probabilities
end % EachAlt

Log_Likelihood = log(Probabilities .^ (DATA.Y~=0)) .* DATA.Y;	% Calculate the log of the likelihood of each alternative in each case
LL = - sum(sum(Log_Likelihood,1),2);							% Total the Log Likelihood across all cases and alternatives

% Determine whether to calculate derivatives
Calculate_Derivatives = (exist('Best_LL') && LL < Best_LL) || (~exist('Best_LL') && nargout > 1);
HESS = zeros(Num.Vars,Num.Vars);				% HESS is the Hessian matrix for the LogLike 
GRAD = zeros(Num.Cases,Num.Vars);					% GRAD is the Gradient for the LogLike
BHHH = zeros(Num.Vars,Num.Vars);

% GRADIENT AND HESSIAN OF THE LOG LIKELIHOOD
if Calculate_Derivatives
	
	Sigma_Components = zeros(Num.Cases, Num.Alts, Num.Vars);	% Initialize Sigma_Components array
	for EachVar = 1:Num.Vars									% Weight each variable by the probabilities
		Sigma_Components(:,:,EachVar) = Probabilities .* reshape(DATA.X(:,EachVar),Num.Cases,Num.Alts);		
	end % EachVar
	Sigma = sum(Sigma_Components,2);				% Sum across alts of each case's variables weighted by the probabilities
	Sigma_Reshape = permute(Sigma,[3 1 2]);			% Reshape Sigma to (Vars, Cases) before looping to improve calculation speed
	
	if Module.RepeatedChoice
		for A = 1:Num.Alts
			AltVars = DATA.X((1:Num.Cases)+((A-1)*Num.Cases),:)';		% Identify the variables for this Alt (V,C)
			Hess_Factor = (AltVars - Sigma_Reshape) ...						% Variables minus Sigma
						.* repmat(Probabilities(:,A)',[Num.Vars 1]) ...		% * Prob (V,C)	
						.* repmat(sum(DATA.Y,2)',[Num.Vars 1]);				% * sum over alts of chosen
			HESS = HESS + (AltVars * Hess_Factor');					% Sum the outer product of the Hess_Factor * Prob
			for V = 1:Num.Vars
				GRAD(:,V) = GRAD(:,V) - AltVars(V,:)' .* (DATA.Y(:,A) - (sum(DATA.Y,2) .* Probabilities(:,A)));	% Sum up the Gradient
			end
		end % A
	else
		for A = 1:Num.Alts
			AltVars = DATA.X((1:Num.Cases)+((A-1)*Num.Cases),:)';		% Identify the variables for this Alt (V,C)
			Hess_Factor = (AltVars - Sigma_Reshape) ...						% Variables minus Sigma
						.* repmat(Probabilities(:,A)',[Num.Vars 1]);		%  		* Prob (V,C)
			HESS = HESS + (AltVars * Hess_Factor');					% Sum the outer product of the Hess_Factor * Prob
			for V = 1:Num.Vars
				GRAD(:,V) = GRAD(:,V) - AltVars(V,:)' .* (DATA.Y(:,A) - Probabilities(:,A));	% Sum up the Gradient
			end
		end % A
	end % Module.RepeatedChoice
	BHHH = GRAD' * GRAD;
	GRAD = sum(GRAD,1)';

	GRAD = Keys.Connector * GRAD;		% Stacks the gradient among connected variables variables
	HESS = Keys.Connector * HESS;		% Stacks the hessian rows among connected variables variables
	HESS = Keys.Connector * (HESS');		% Stacks the hessian columns among connected variables variables
	BHHH = Keys.Connector * BHHH;		% Stacks the hessian rows among connected variables variables
	BHHH = Keys.Connector * (BHHH');		% Stacks the hessian columns among connected variables variables
	
	Grad_to_Show = GRAD' .* DATA.Max_Adj.A;			% Unscale Gradient using DATA.Max_Adj
	
	GRAD = GRAD(Keys.Free);		% Restrict gradient reported by the function to only flexible variables
	HESS = HESS(:, Keys.Free);	% Constrict Hessian to Free Vars
	HESS = HESS(Keys.Free, :);
	BHHH = BHHH(:, Keys.Free);	% Constrict BHHH to Free Vars
	BHHH = BHHH(Keys.Free, :);
				
end % Calculate_Derivatives

% Unscaling of BETA for Reporting
BETA_to_show = BETA./DATA.Max_Adj.A;				% Unscale BETA parameters using DATA.Max_Adj
Current_Parameters.BETA = BETA_to_show;

% Results Output to Screen and Log File
if Prefs.VerboseOutput
	relay ('\nITERATION NUM: %d \n',ITERATION);
	relay ('%s',repmat(' ',1,length(Var_Names))); 
	relay ('\tBETA:    \t'); 
	if Calculate_Derivatives
		relay ('Gradient:');
	end
	relay ('\n'); 
	for EachVar = 1:Num.Vars
		relay ('%s\t', Var_Names.BETA(EachVar,:));
		relay ('%9.4g\t',BETA_to_show(EachVar));	
		if Calculate_Derivatives
			relay ('%9.4g', Grad_to_Show(1,EachVar));
		end
		relay ('\n');
	end % EachVar
	relay ('LL: - %0.8g\n',LL);
end

% Check for log file writing
if LogCheck == ftell(LogFile)
	fprintf('RECORDING ERROR : LOG FILE WRITE BYTE UNCHANGED\n');
else
	LogCheck = ftell(LogFile);
end

%-----------------------------------------------------------------------
function [LL, GRAD, HESS, BHHH] = LL_MNL_XRM (PAR_open, PAR_lock, Best_LL)
% LL_MNL_XRM (PAR_open, PAR_lock, Best_LL)
% 	This function calculates the (negative of) log likelihood of a standard
%	multinomial logit model, given a set of parameters. It is compatible with
%	CaseID/AltID (DATA.X) format data, as well as DATA.R and DATA.M
%	data formats. The negative is returned to be compatible with
%	standard minimization algorithms, since the goal is to maximize LL.
%
% INPUT:
%	PAR_open	Col Vector	The unknown parameters, across which Log Likelihood is maximized
%	PAR_lock	Col Vector	Constrained parameters, across which Log Likelihood is maximized
%	Best_LL		Scalar		(optional) The best log likelihood found so far. This is used as
%							a trigger to skip the calculation of the gradient and hessian
%							if this iteration is not an improvement.
% OUTPUT:
%	LL			Scalar		The (negative of) log likelihood
%	GRAD		Col Vector	The vector of first order partial derivatives
%	HESS		Matrix		The matrix of second partial derivatives
%	BHHH		Matrix		The sum of the outer product of the gradient at each case
 
 
global LogFile LogCheck
global Prefs
global ITERATION 
global Var_Names 
global Current_Parameters
global Module
global DATA 
global Num 
global Connect 
global Keys 

ITERATION = ITERATION + 1;

BETA = create_working_PAR (PAR_open, PAR_lock, Keys.Unlock, Keys.Connector);

% Initialize output variables
LL 		= 0;
GRAD = zeros(size(BETA));
HESS = zeros(length(BETA));
BHHH = zeros(length(BETA));

% Break relevant data out of structure for speed
Origin = DATA.O;
Num_Origins = size(DATA.O,2);
Num_Alts = Num.Alts;
Num_Vars = Num.Vars;
CHOSEN = DATA.Y';
DATA_M = DATA.M;

% Establish case weights
if isfield(DATA,'W')
	Weight = DATA.W;
else
	Weight = true(Num.Cases,1);
end

% Establish standard (X) variables
if isfield(DATA,'X')
	X_AVC = permute( reshape(DATA.X, Num.Cases, Num.Alts, []) , [2 3 1]);
else
	X_AVC = zeros(Num.Alts,0,Num.Cases);
end
Num_X_Var = size(X_AVC,2);
	

% Build initial case data matrix
Baseline_DATA_M = [DATA_M{1}(:,:,1)];
for m = 2:Num_Origins
	Baseline_DATA_M = [Baseline_DATA_M DATA_M{m}(:,:,1)];
end

Case_DATA = [X_AVC(:,:,1) DATA.R Baseline_DATA_M];
Num_Prev_Cols = size(X_AVC,2) + size(DATA.R,2);

for m = 1:Num_Origins
	M_Cols{m} = (1:size(DATA_M{m},2)) + Num_Prev_Cols;
	Num_Prev_Cols = Num_Prev_Cols + size(DATA_M{m},2);
end

% Cycle through all cases
for C = 1:Num.Cases
	
	% Pull in Changing Pieces
	Case_DATA(:,1:Num_X_Var) = X_AVC(:,:,C);
	for m = 1:Num_Origins
		Case_DATA(:,M_Cols{m}) = DATA_M{m}(:,:,Origin(C,m));
	end
	Y = CHOSEN(:,C);
	
			
	% Build Utility, Cum_Utility and Probabilities Matrices
	Utility = Case_DATA * (BETA');				% (A,1)
	Utility = exp(Utility);			
	Cum_Utility = sum(Utility);					% Calculate total exp_utility in each case for the denominator
	Probabilities = Utility ./ Cum_Utility;		% Find the logit probabilities
	
	% Total the Log Likelihood across all cases and alternatives
	if Module.BinaryChoice
		LL = LL - log(Probabilities(Y)) .* Weight(C);
	else % ~Module.BinaryChoice
		LL = LL - sum(log(Probabilities.^Y)) .* Weight(C);
	end
	if isinf(LL); break; end;
	
end % C


% Determine whether to calculate derivatives
Calculate_Derivatives = (exist('Best_LL') && LL < Best_LL) || (~exist('Best_LL') && nargout > 1);
HESS = zeros(Num.Vars,Num.Vars);				% HESS is the Hessian matrix for the LogLike 
GRAD = zeros(Num.Cases,Num.Vars);			% GRAD is the Gradient for the LogLike

if Calculate_Derivatives
	for C = 1:Num.Cases
		% Pull in Changing Pieces
		Case_DATA(:,1:Num_X_Var) = X_AVC(:,:,C);
		for m = 1:Num_Origins
			Case_DATA(:,M_Cols{m}) = DATA_M{m}(:,:,Origin(C,m));
		end
		Y = CHOSEN(:,C);
	
		% Build Utility, Cum_Utility and Probabilities Matrices
		Utility = Case_DATA * (BETA');				% (A,1)
		Utility = exp(Utility);			
		Cum_Utility = sum(Utility);					% Calculate total exp_utility in each case for the denominator
		Probabilities = Utility ./ Cum_Utility;		% Find the logit probabilities

		% GRADIENT AND HESSIAN OF THE LOG LIKELIHOOD
		Sigma = Probabilities' * Case_DATA;
		for V = 1:Num_Vars
			Hess_Factors(:,V) = (Case_DATA(:,V) - Sigma(V)) .* Probabilities;
		end
		%	Hess_Factors = (Case_DATA - repmat(Sigma,[Num_Alts 1])) ...
		%				.* repmat(Probabilities,[1 Num_Vars]) ...
		%				.* sum(Y);
		HessA = (Hess_Factors' * Case_DATA);
		HessB = HessA * Weight(C);
		HESS = HESS + HessB;
		GRAD(C,:) = GRAD(C,:) - ((Y - Probabilities) .* Weight(C))' * Case_DATA;	% Sum up the Gradient
	end % C	

	BHHH = GRAD' * GRAD;
	GRAD = sum(GRAD,1)';

	GRAD = Keys.Connector(1:Num.Vars,1:Num.Vars) * GRAD;		% Stacks the gradient among connected variables variables
	HESS = Keys.Connector(1:Num.Vars,1:Num.Vars) * HESS;		% Stacks the hessian rows among connected variables variables
	HESS = Keys.Connector(1:Num.Vars,1:Num.Vars) * (HESS');	% Stacks the hessian columns among connected variables variables
	BHHH = Keys.Connector(1:Num.Vars,1:Num.Vars) * BHHH;		% Stacks the hessian rows among connected variables variables
	BHHH = Keys.Connector(1:Num.Vars,1:Num.Vars) * (BHHH');	% Stacks the hessian columns among connected variables variables
	
	Grad_to_Show = GRAD' .* DATA.Max_Adj.A;			% Unscale Gradient using DATA.Max_Adj
	
	GRAD = GRAD(Keys.Free(1:Num.Vars));		% Restrict gradient reported by the function to only flexible variables
	HESS = HESS(:, Keys.Free(1:Num.Vars));	% Constrict Hessian to Free Vars
	HESS = HESS(Keys.Free(1:Num.Vars), :);
	BHHH = BHHH(:, Keys.Free(1:Num.Vars));	% Constrict BHHH to Free Vars
	BHHH = BHHH(Keys.Free(1:Num.Vars), :);
end % nargout

% Unscaling of BETA for Reporting
BETA_to_show = BETA./DATA.Max_Adj.A;				% Unscale BETA parameters using DATA.Max_Adj
Current_Parameters.BETA = BETA_to_show;

% Results Output to Screen and Log File
if Prefs.VerboseOutput
	relay ('\nITERATION NUM: %d \n',ITERATION);
	relay ('%s',repmat(' ',1,length(Var_Names))); 
	relay ('\tBETA:    \t'); 
	if Calculate_Derivatives
		relay ('Gradient:');
	end
	relay ('\n'); 
	for EachVar = 1:Num.Vars
		relay ('%s\t', Var_Names.BETA(EachVar,:));
		relay ('%9.4g\t',BETA_to_show(EachVar));	
		if Calculate_Derivatives
			relay ('%9.4g', Grad_to_Show(1,EachVar));
		end
		relay ('\n');
	end % EachVar
	relay ('LL: - %0.8g\n',LL);
end

% Check for log file writing
if LogCheck == ftell(LogFile)
	fprintf('RECORDING ERROR : LOG FILE WRITE BYTE UNCHANGED\n');
else
	LogCheck = ftell(LogFile);
end

%-----------------------------------------------------------------------
function [LL, GRAD, HESS, BHHH] = LL_NL (PAR_open, PAR_lock, Nesting, Best_LL)
% LL_MNL_X (PAR_open, PAR_lock, Best_LL)
% 	This function calculates the (negative of) log likelihood of a standard
%	multinomial logit model, given a set of parameters. It is optimized
%	for CaseID/AltID format data, (DATA.X) and will not function with any
%	other data format. The negative is returned to be compatible with
%	standard minimization algorithms, since the goal is to maximize LL.
%
% INPUT:
%	PAR_open	Col Vector	The unknown parameters, across which Log Likelihood is maximized
%	PAR_lock	Col Vector	Constrained parameters, across which Log Likelihood is maximized
%	Nesting		LogicMatrix	A square matrix of logical indicators for the nesting structure
%							Each row has a single mark indicating the parent nest location
%							Each column has a set of marks indicating the nests' children
%	Best_LL		Scalar		(optional) The best log likelihood found so far. This is used as
%							a trigger to skip the calculation of the gradient and hessian
%							if this iteration is not an improvement.
% OUTPUT:
%	LL			Scalar		The (negative of) log likelihood
%	GRAD		Col Vector	The vector of first order partial derivatives
%	HESS		Matrix		The matrix of second partial derivatives
%	BHHH		Matrix		The sum of the outer product of the gradient at each case
 
global ITERATION Hess_LL Var_Names LogFile LogCheck Current_Parameters
global Module DATA Num Connect Keys Prefs

error_check(Num.Vars==size(DATA.X,2), ...
	'ERROR: The Nested Logit Model does not currently accept any observed variables that are not in Case-Alt format.');

ITERATION = ITERATION + 1;

s=warning('off','all'); % Temporary warning supression to ensure repeated warning displays don't slow us down

[BETA, THETA, MU] = create_working_PAR (PAR_open, PAR_lock, Keys.Unlock, Keys.Connector);

Mu = ones(1,Num.Nests);
Mu(Num.Alts+1:end-1) = MU;

% Initialize array variables
Utility =		zeros(Num.Cases, Num.Alts);
Cum_Utility =	zeros(Num.Cases, 1       );
Probabilities = zeros(Num.Cases, Num.Nests);
Probabilities(:, Num.Nests) = 1;				% Probability of top level nest is 1
	
% Build Utility Matrices
Utility = DATA.X * (BETA');
Utility = reshape(Utility, Num.Cases, Num.Alts);

Utility(~DATA.Z) = -Inf;		% Unavailable alternatives get -Inf observed utility

% Out of bounds correction:
%	if any utility value is greater than about 700, or less than -700,
%	 then the exp(Utility) for that item will round to Infinity in Matlab
%	so to protect against Infinite positive utility, the utility array is rescaled 
%	(only arithemtic difference in utility matters so this change does not alter the solution) 
%	but it does add unnecessary computation if it is not needed
%	Correction: too small out-of-bounds is also bad
if max(max(max(Utility))) >= 700 | min(min(min(Utility))) <= -700
	Utility = Utility - repmat(max(Utility,[],2),[1,Num.Alts,1]);
end

% From the bottom nests up, calculate the utilities of nests
for N = Num.Alts+1:Num.Nests
    if Mu(N) == 0
        Utility(:, N) = max(Utility(:,Nesting(:,N)),[],2);
    else
        Child_Utility = Utility(:,Nesting(:,N)) / Mu(N);
        % Out of bounds correction:
        if max(max(max(Child_Utility))) >= 700 | min(min(min(Child_Utility))) <= -700
            Nest_Utility_Adj = max(Child_Utility,[],2);
            Nest_Utility_Adj(isinf(Nest_Utility_Adj)) = 0;
        	Child_Utility = Child_Utility - repmat(Nest_Utility_Adj,[1,size(Child_Utility,2),1]);
        else
            Nest_Utility_Adj = zeros(Num.Cases,1);
        end
        Utility(:, N) = (log(sum(  exp(Child_Utility)  ,2))+ Nest_Utility_Adj) * Mu(N);
    end
end % N

% From the top down, calculate probabilities of nests and alternatives
for N = Num.Nests:-1:Num.Alts+1	
	Nest_Children = Nesting(:,N);
	for EachChild = find(Nest_Children)'
        Net_Utility = Utility(:, EachChild)-Utility(:, N);
        Net_Utility(isnan(Net_Utility)) = 0;
        if Mu(N)==0
			Cond_Probabilities(:, EachChild) = (Net_Utility==0);
            Probabilities(:, EachChild) = (Net_Utility==0) .* Probabilities(:, N);
        else
			Cond_Probabilities(:, EachChild) = exp(  Net_Utility/Mu(N)  );
			Probabilities(:, EachChild) = Cond_Probabilities(:, EachChild) .* Probabilities(:, N);
		end
    end % EachChild
end

% Calculate the log of the likelihood of each alternative in each case
Log_Likelihood = log(Probabilities(:,1:Num.Alts) .^ (DATA.Y~=0)) .* DATA.Y;	

% Create finite large neagtive loglikelihoods for available alternatives that have zeroed
Log_Likelihood( isfinite(Utility(:,1:Num.Alts)) & (Probabilities(:,1:Num.Alts)==0) ) = -1e25;

% Total the Log Likelihood across all cases and alternatives
LL = - sum(sum(Log_Likelihood,1),2);	

% Determine whether to calculate derivatives
Calc_Gradient = (exist('Best_LL') && LL < Best_LL) || (~exist('Best_LL') && nargout > 1);

% Create placeholders if not generating gradients
if ~Calc_Gradient
	BHHH = zeros(Num.Vars,Num.Vars);		% the Hessian matrix for the LogLike 
	GRAD = zeros(Num.Vars,1);					% the Gradient for the LogLike
end

%%% GRADIENT and BHHH OF THE LL %%%
if Calc_Gradient
	
	Grad_BETA = zeros(Num.Cases,Num.Vars);			% Grad_LL is the Gradient for the LogLike
	Grad_MU	  =	zeros(Num.Cases,Num.Nests);
	
	% Gradient of the BETA Parameters
	for V = 1:Num.Vars
		dUtility_dBETA = zeros(Num.Cases,Num.Nests);
		
		% dUtility_dBETA of the elemental alternatives
		for A = 1:Num.Alts
			dUtility_dBETA(1:Num.Cases,1:Num.Alts) = reshape(DATA.X(:,V),Num.Cases,Num.Alts);
		end
		
		% dUtility_dBETA of the nests
		for N = Num.Alts+1:Num.Nests
			Nest_Children = Nesting(:,N);
			for B = find(Nest_Children)' % Each Child (Baby) 
				dUtility_dBETA(:,N) = dUtility_dBETA(:,N) + (Cond_Probabilities(:,B) .* dUtility_dBETA(:,B)); 
				Grad_Component = - (DATA.Y_NL(:,B) - Cond_Probabilities(:,B)) .* dUtility_dBETA(:,B) ./ Mu(N);
				Grad_Component = Grad_Component .* DATA.Y_NL(:,N);
				Grad_Component(isnan(Grad_Component)) = 0;
				Grad_BETA(:,V) = Grad_BETA(:,V) + Grad_Component;
			end % B	
		end % N
	end % V
	
	
	% Gradient of the MU Parameters
	for m = Num.Alts+1:Num.Nests-1
		
		dUtility_dMU = zeros(Num.Cases,Num.Nests);

		% dUtility_dMU of the Children of the Mu
		dUtility_dMU(:,Nesting(:,m)) = -Utility(:,Nesting(:,m)) / Mu(m);
	
		% dUtility_dMU of the Nest of Mu
		PeerPart = Cond_Probabilities(:,Nesting(:,m)) .* Utility(:,Nesting(:,m));
		PeerPart(isnan(PeerPart)) = 0;
		PeerPart = sum(PeerPart,2);
		dUtility_dMU(:,m) = ( Utility(:,m) - PeerPart ) ./ Mu(m);

		% Grad_MU (dLLdMU) of the Nest of Mu
		ChildPart = (DATA.Y_NL(:,Nesting(:,m)) - Cond_Probabilities(:,Nesting(:,m))) .* dUtility_dMU(:,Nesting(:,m));
		ChildPart(~isfinite(ChildPart)) = 0;
		ChildPart = sum(ChildPart,2);
		Grad_MU(:,m) = Grad_MU(:,m) - ChildPart .* DATA.Y_NL(:,m) ./ Mu(m);		
				
		% Cascading to the Root Node
		N_back = m;		% Records the child from the cascade point towards the Mu
		N = find(Nesting(m,:));	% Identify the parent of m

		while N <= Num.Nests
			% dUtility_dMU of the cascade point
			dUtility_dMU(:,N) = Cond_Probabilities(:,N_back) .* dUtility_dMU(:,N_back);

			% Grad_MU (dLLdMU) of the cascade point
			ChildPart = (DATA.Y_NL(:,Nesting(:,N)) - Cond_Probabilities(:,Nesting(:,N))) .* dUtility_dMU(:,Nesting(:,N));
			ChildPart(~isfinite(ChildPart)) = 0;
			ChildPart = sum(ChildPart,2);
			
			Grad_MU(:,m) = Grad_MU(:,m) - ChildPart .* DATA.Y_NL(:,N) ./ Mu(N);		
			
			% Step up to the next cascade point towards the root
			N_back = N;
			N = find(Nesting(N,:));	% Identify the parent of N
		end
	
	end

	% Delete unused portions of Grad_MU
	Grad_MU(:,1:Num.Alts) = [];
	Grad_MU(:,end) = [];

	GRAD_Complete = [Grad_BETA, Grad_MU];
	GRAD_Complete = GRAD_Complete';
	GRAD_Complete =  Keys.Connector * GRAD_Complete;	% Stacks the gradient among connected variables
	
	BHHH = GRAD_Complete * (GRAD_Complete');
	BHHH = BHHH(Keys.Free,Keys.Free);
	
	GRAD_Complete = sum(GRAD_Complete,2);
	GRAD = GRAD_Complete(Keys.Free);	% Restrict gradient reported by the function to only flexible variables
	
	Report_Grad_BETA = GRAD_Complete(1:Num.Vars)' .* DATA.Max_Adj.A;		% Unscale Gradient using DATA.Max_Adj
	Report_Grad_MU = GRAD_Complete(Num.Vars+1:end);	
	
end %%% END OF GRADIENT OF THE LL %%%
	
% No analytic hessian is calculated for NL models
HESS = BHHH;

% Unscaling of BETA for Reporting
BETA_to_show = BETA./DATA.Max_Adj.A;	% Unscale BETA parameters using DATA.Max_Adj

% Save BETA to Current_Parameters global for restartability
Current_Parameters.BETA = BETA_to_show;
Current_Parameters.MU = MU;

% Results Output to Screen and Log File every 5 seconds
global Time_IterateBegin
if Prefs.VerboseOutput 
	relay ('\nITERATION NUM: %d \n',ITERATION);
	relay ('Variable: \tBETA:    '); 
	if Calc_Gradient
		relay ('\tGradient:'); 
	end
	relay ('\n'); 
	for EachVar = 1:Num.Vars
		relay ('%10s\t', Var_Names.BETA(EachVar,:));
		relay ('%9.4g\t',BETA_to_show(EachVar));	
		if Calc_Gradient
			relay ('%9.4g',Report_Grad_BETA(EachVar));
		end
		relay ('\n');	
	end % EachVar
	for EachLogSum = 1:Num.LogSums
		relay ('%10s\t', Var_Names.MU(EachLogSum,:));
		relay ('%9.4g\t',MU(EachLogSum));	
		if Calc_Gradient
			relay ('%9.4g',Report_Grad_MU(EachLogSum));
		end
		relay ('\n');	
	end
		
	relay ('LL: - %0.10g\n',LL);
	
	% Check for log file writing
	if LogCheck == ftell(LogFile)
		fprintf('RECORDING ERROR : LOG FILE WRITE BYTE UNCHANGED\n');
	else
		LogCheck = ftell(LogFile);
	end

end
warning(s);

%-----------------------------------------------------------------------
function [LL, Grad_MXL, Hess_MXL, BHHH] = LL_MXL (PAR_open, PAR_lock, Std_Draws, Best_LL)
% This function calculates the (negative of) log likelihood of a mixed logit model.
% INPUT:
%	PAR_open		Row Vector	The unknown variables
%	PAR_lock 		Row Vector	Parameters that are constrained
%	Dictionary		2RowMatrix	Dictionary on how to construct the BETA_plain 
%	Dist.All_Types	String		One character per variable for each type: 
%								Characters in order as varaibles appear in the Dictionary dictionary
%								See top-level help for available types
%	Random_Vars		Vector		Dictionary of locations for Random Variables in BETA_plain
%	Random_Var_TypesString		Dist.All_Types truncated to Random variables only
%	NonRandom_Vars	Vector		Dictionary of locations for NonRandom Variables in BETA_plain
%	Num.Vars		Scalar		Number of explanitory variables
%	Num.Alts		Scalar		Number of alternatives available across all cases
%	Num.Cases		Scalar		Number of cases
%	Num.CaseAlts	Scalar		Number of Case-Alts (Rows) in the original flat data file
%	DATA.Y			Array		(Case, Alt)
%	DATA			Array		(Case, Alt, Variable)
%	DATA.Max_Adj			Row Vector
%	Draws			Array		(Num_Random_Vars, Num.Draws(1), Num.Cases)
%	Status_Text		String		A string to display at each iteration noting the type of operation being calculated
% GLOBALS:
%	ITERATION		Scalar		The iteration number, used for reporting interim results
%	LogFile			Code Number	Developed when the log file is opened/created
%	LogCheck		Scalar		Current Byte location in the Log File, used to confirm write commands
%	Var_Names		StringArray	The names of variables, used in reporting results

global ITERATION LogFile LogCheck Var_Names Current_Parameters 
global Module DATA Num Connect Keys Prefs
global Dist
global Status_Text

error_check(Num.Vars==size(DATA.X,2), ...
	'ERROR: The Mixed Logit Model does not currently accept any observed variables that are not in Case-Alt format.');

ITERATION = ITERATION + 1;						% Count the iterations
relay ('\nITERATION NUM: %d, %s\n', ITERATION, Status_Text);		% and display the number on the log

% Construct the BETA_plain from open and lock components
[BETA, THETA] = create_working_PAR (PAR_open, PAR_lock, Keys.Unlock, Keys.Connector);


% Initialize LogNormal_Spots, which identifies the LogNormal spots within the random variables
LogNormal_Spots = [];

% Create negatives of Duos where applicable

Std_Draws_DVC = Std_Draws.DVC;

% Duo_Mirror
if Prefs.X_Mirror	
	for V = 1:Num.Random_Vars		% Every alternate variable
		if THETA(Dist.Random_Vars(V)) < 0
			Std_Draws_DVC(:,V,DATA.G.Duos) = -Std_Draws_DVC(:,V,DATA.G.Duos);	% Mirror draws for the other person
		end
	end
end % Prefs.X_Mirror



% Apply transformations to standard random variable draws to create draw realizations
for EachRandomVar = 1:Num.Random_Vars							% For each random variable
	ThisLocation = BETA (Dist.Random_Vars(EachRandomVar));		% The location parameter for this random variable
	ThisSpread   = THETA(Dist.Random_Vars(EachRandomVar));		% The spread parameter for this random variable
	switch Dist.Random_Var_Types(EachRandomVar)	
	case {'N','n','M','m'}						% N, M: Normal
		BETA_Draws_DVC(:,(EachRandomVar),:) = (Std_Draws_DVC(:,(EachRandomVar),:) * (ThisSpread)) + ThisLocation;
	case {'L','l','O','o'}						% L, O: LogNormal
		BETA_Draws_DVC(:,(EachRandomVar),:) = exp((Std_Draws_DVC(:,(EachRandomVar),:) * (ThisSpread)) + ThisLocation);
		LogNormal_Spots(end+1) = EachRandomVar;
	case {'E','e','V','v'}						% E, V: Negative LogNormal
		BETA_Draws_DVC(:,(EachRandomVar),:) = -exp((Std_Draws_DVC(:,(EachRandomVar),:) * (ThisSpread)) + ThisLocation);
		LogNormal_Spots(end+1) = EachRandomVar;
	case {'G','g','H','h'}						% G, H: Gumbel
		BETA_Draws_DVC(:,(EachRandomVar),:) = (Std_Draws_DVC(:,(EachRandomVar),:) * (ThisSpread)) + ThisLocation;
	case {'U','u','W','w'}						% U, W: Uniform
		BETA_Draws_DVC(:,(EachRandomVar),:) = (Std_Draws_DVC(:,(EachRandomVar),:) * (ThisSpread)) + ThisLocation;
	case {'T','t','R','r'}						% T, R: Triangle
		BETA_Draws_DVC(:,(EachRandomVar),:) = (Std_Draws_DVC(:,(EachRandomVar),:) * (ThisSpread)) + ThisLocation;
	case {'Y'}									% Y: Positive Socialization
		BETA_Draws_DVC(:,(EachRandomVar),:) = (Std_Draws_DVC(:,(EachRandomVar),:) * (ThisSpread)) + ThisLocation;
	case {'Z'}									% Z: Negative Socialization
%		for D=1:Num.Draws(1)
%			Std_Draws_DVC(D,EachRandomVar,:) = Std_Draws_DVC(D,EachRandomVar,:) .* DATA.G.Duos;
%			Std_Draws.KRD(:,EachRandomVar,D) = Std_Draws.KRD(:,EachRandomVar,D) .* DATA.G.DuosK;
%		end
		BETA_Draws_DVC(:,(EachRandomVar),:) = (Std_Draws_DVC(:,(EachRandomVar),:) * (ThisSpread)) + ThisLocation;		
	end

end % EachRandomVar





BETA_Points_xVx = zeros(1,Num.NonRandom_Vars);
for EachNonRandomVar = 1:Num.NonRandom_Vars
	BETA_Points_xVx(1,EachNonRandomVar) = BETA(Dist.NonRandom_Vars(EachNonRandomVar));
end

% Initialize array variables
Utility_DAC		= zeros(Num.Draws(1), Num.Alts, Num.Cases);
Cum_Utility 	= zeros(Num.Cases, 1       , Num.Draws(1));
Probabilities 	= zeros(Num.Cases, Num.Alts, Num.Draws(1));

% Sort Random and Non-Random parameters
DATA_VAC_Random = DATA.VAC(Dist.Random_Vars,:,:);
DATA_VAC_NonRandom = DATA.VAC(Dist.NonRandom_Vars,:,:);

%	fprintf('...Utilities\n');

% Calculate Utilities
for EachCase = 1:Num.Cases
	
	% Calculate the fixed portion of Utility
	Utility_DAC(1,:,EachCase) = BETA_Points_xVx(1,:,1) * DATA_VAC_NonRandom(:,:,EachCase);
	
	% Replicate the fixed portion across all draws
	Utility_DAC(:,:,EachCase) = Utility_DAC(ones(1,Num.Draws(1)),:,EachCase);
	
	% Add the random portion of observed utility
	Utility_DAC(:,:,EachCase) = Utility_DAC(:,:,EachCase) + ...
								(BETA_Draws_DVC(:,:,EachCase) * DATA_VAC_Random(:,:,EachCase));
	
	
%	Utility_DAC(:,:,EachCase) = (BETA_Draws_DVC(:,:,EachCase) * DATA_VAC_Random(:,:,EachCase)) + ...
%						repmat( (BETA_Points_xVx(1,:,1) * DATA_VAC_NonRandom(:,:,EachCase)) ,Num.Draws(1),1);	
end

% Error checking for infinite of NaN values
warn_check(~any(any(any(isnan(Utility_DAC)))),'     Caution: This iteration has %0.0f NaN instances in Utility. \n', sum(sum(sum(isnan(Utility_DAC)))));	
warn_check(~any(any(any(isinf(Utility_DAC)))),'     Caution: This iteration has %0.0f Inf instances in Utility. \n', sum(sum(sum(isinf(Utility_DAC)))));	

Utility = permute(Utility_DAC,[3 2 1]);											% Change shape to (Cases, Alts, Draws)

Unavailable = zeros(size(DATA.Z));
Unavailable(~DATA.Z) = -Inf;

% Set unavailable utilitys to -Inf
for EachDraw = 1:Num.Draws(1)													
	Utility(:,:,EachDraw) = Utility(:,:,EachDraw) + Unavailable;		% Change to -Inf for unavailable choices
end																				

% Out of bounds correction:
%	if any utility value is greater than about 700, then the exp(Utility) for that item will round to Infinity in Matlab
%	so to protect against Infinite positive utility, the utility array is rescaled 
%	(only arithemtic difference in utility matters so this change does not alter the solution, 
%	but it does add unnecessary computation if it is not needed)
%	Correction: too small out-of-bounds is also bad
if max(max(max(Utility))) >= 700 | min(min(min(Utility))) <= 700
	Utility = Utility - repmat(max(Utility,[],2),[1,Num.Alts,1]) + 700;
end

Exp_Utility = exp(Utility);		

warn_check(~any(any(any(isnan(Exp_Utility)))),'     Caution: This iteration has %0.0f NaN instances in Exp_Utility. \n', sum(sum(sum(isnan(Exp_Utility)))));	
warn_check(~any(any(any((Exp_Utility==Inf)))),'     Caution: This iteration has %0.0f +Inf instances in Exp_Utility. \n', sum(sum(sum(isinf(Exp_Utility)))));	

% Calculate exponential of utility to ensure positive values
%for EachDraw = 1:Num.Draws(1)													
%	Exp_Utility(:,:,EachDraw) = Exp_Utility(:,:,EachDraw) .* DATA.AVAIL;		% Change to zero for unavailable choices
%end																				
Exp_Utility(isnan(Exp_Utility)) = 0;											% Set NaN's that arise from -Inf * 0 calculations to zero

%	fprintf('...Probabilities\n');

Cum_Utility = sum(Exp_Utility,2);												% Cumulative utility across altervatives in each case (denominator)

warn_check(~any(any(any(isnan(Cum_Utility)))),'     Caution: This iteration has %0.0f NaN instances in Cum_Utility \n', sum(sum(sum(isnan(Cum_Utility)))));	
warn_check(~any(any(any(isinf(Cum_Utility)))),'     Caution: This iteration has %0.0f Inf instances in Cum_Utility \n', sum(sum(sum(isinf(Cum_Utility)))));	
warn_check(~any(any(any((Cum_Utility==0)))),'     Caution: This iteration has %0.0f zero instances in Cum_Utility \n', sum(sum(sum((Cum_Utility==0)))));	

s = warning('off', 'MATLAB:divideByZero');

for EachAlt = 1:Num.Alts
	Probabilities(:,EachAlt,:) = Exp_Utility(:,EachAlt,:)./Cum_Utility(:,1,:);	% Calculate Standard Logit Probabilities (Case, Alt, Draw)
end % EachAlt

warning(s);

warn_check(~any(any(any(isnan(Probabilities)))),'     Caution: This iteration has %0.0f NaN instances in Probabilities \n', sum(sum(sum(isnan(Probabilities)))));	
warn_check(~any(any(any(isinf(Probabilities)))),'     Caution: This iteration has %0.0f Inf instances in Probabilities \n', sum(sum(sum(isinf(Probabilities)))));

% Fix out of range probabilities
Probabilities(Exp_Utility==Inf) = 1;
Probabilities(Exp_Utility==0) = 0;
Probabilities(isnan(Probabilities)) = 0;

% Rescale for overage
Num_Clashing_Infinites = sum(sum(sum(Probabilities,2)>1.1));
if Num_Clashing_Infinites > 0
	Probabilities = Probabilities ./ repmat(sum(Probabilities,2),[1,Num.Alts,1]);
	relay('     Caution: This iteration contains %0.0f cases with multiple out-of-bounds infractions\n',Num_Clashing_Infinites);
end % if


%	fprintf('...Likelihoods\n');

% Calculate Likelihoods from Probabilities
if Module.BinaryChoice == 1
	Likelihoods = Probabilities;
	Likelihoods(~DATA.Y_D) = 1;		% This is much faster than the mathematically correct power calculation
else									% When BinaryChoice is not applicable, the power calculation is used
	for EachDraw = 1:Num.Draws(1)
		Likelihoods(:,:,EachDraw) = (Probabilities(:, :, EachDraw) .^ DATA.Y);		% Likelihoods(Case, Alt, Draw)
	end % EachDraw
end % if

% Error Checking
if any(any(any(Likelihoods == 0)))	% Check for zero values (this is an underflow problem and not a fatal error)
	relay('     Caution: This iteration contains %0.0f likelihoods rounded to zero\n',sum(sum(sum(Likelihoods == 0,1),2),3));
%	relay('         in %0.0f seperate cases\n',sum(sum(0~=(sum(Likelihoods == 0,1)),2),3));
end
	if any(any(any(isnan(Likelihoods))))	% Check for NaN values (this is an over/underflow problem and not a fatal error)
		relay('     Caution: This iteration contains %0.0f NaN likelihoods',sum(sum(sum(isnan(Likelihoods),1),2),3));
		relay(' in %0.0f seperate cases\n',sum(sum(0~=(sum(isnan(Likelihoods),1)),2),3));
	end
Likelihoods(isnan(Likelihoods)) = 0;											% Set NaN's that arise to zero

% Roll up likelihood across alts, draws, and cases
Likelihoods = prod(Likelihoods,2);					% Multiple across alts to make Likelihoods(Case, 1, Draw)

if Prefs.X_Population == 0
	% For Panel Data
	if Module.Panel
		Likelihoods_G = zeros(DATA.G.Num,1,Num.Draws(1));
		for EachGroup = 1:DATA.G.Num					% Collapse Likelihoods by Groups
			Likelihoods_G(EachGroup,1,:) = prod(Likelihoods(DATA.G.Begins(EachGroup):DATA.G.Ends(EachGroup),1,:),1);
		end % EachGroup
		Likelihoods = Likelihoods_G;
	end	
	Log_Likelihoods = log(mean(Likelihoods,3));		% Average across draws within each group or case, then take the log
	LL = -sum(Log_Likelihoods);						% Sum LL over all cases in the sample, negative for minimize routine
else % Population module
	Likelihoods = sum(log(Likelihoods),1);			% Condense the whole sample into one likelihood
	RangeFactor = max(Likelihoods);					% Find an adjustment factor to avaiod out-of-range errors
	Likelihoods = Likelihoods - RangeFactor;
	Likelihoods = exp(Likelihoods);					% Un-log the adjusted values before taking the mean
	LL = log(mean(Likelihoods,3));					% Average across draws from the whole sample, then (re)take the log
	LL = LL + RangeFactor;
	LL = -LL;
	Likelihoods = repmat(Likelihoods,[DATA.G.Num 1 1]);	% Re-inflate Likelihoods to fill in the gradient section correctly
end	


% % % GRADIENT AND BHHH OF THE LOG LIKELIHOOD % % %

% Determine whether to calculate derivatives
Calc_Gradient = (exist('Best_LL') && LL < Best_LL) || (~exist('Best_LL') && nargout > 1);

if Calc_Gradient									% If number of output arguments is greater than 1, compute gradient and BHHH

	% Organize Meta_DATA
	Order_of_Meta_DATA = zeros(1,Num.Vars*2);
	Order_of_Meta_DATA(1:end) = 1:Num.Vars*2;
	LogNormal_Vars = regexp(Dist.All_Types,'[LlOoEeVv]');			% Find the index locations of the LogNormal variable types

	% Reinflation of Likelihoods and change to weighting values
	Likelihoods = Likelihoods(DATA.G.Cases,1,:);					% Reinflate Likelihoods to (Case, 1, Draw) %!!!!!
	Cum_Likelihoods = sum(Likelihoods,3);							% Creates Cumulative Likelihoods
	Likelihoods = Likelihoods ./ Cum_Likelihoods(:,1,ones(1,Num.Draws(1)));	% Reduces Likelihoods to their relative weights
    Likelihoods(isnan(Likelihoods)) = 0;                            % Kill the NaN likelihoods    
    Likelihoods = Likelihoods(:,ones(1,Num.Alts),:);				% Reinflate Likelihoods to (Case, Alt, Draw)
	Likelihoods = reshape(Likelihoods,Num.K_CaseAlt,Num.Draws(1));	% Reshape Likelihoods to (CaseAlt, Draw)	
	Probabilities = reshape(Probabilities,Num.K_CaseAlt,Num.Draws(1));
	DATA.Y = reshape(DATA.Y,Num.K_CaseAlt,1);
	DATA.Y_D = reshape(DATA.Y_D,Num.K_CaseAlt,Num.Draws(1));
	
	GRAD_G = zeros(Num.Vars+Num.Random_Vars, DATA.G.Num);						% Initialize Grad_Group
	GRAD   = zeros(Num.K_CaseAlt,Num.Vars+Num.Random_Vars);
	
%	fprintf('...Gradient\n');


    % Gradient of Locations
	YP = sum((DATA.Y_D - Probabilities) .* Likelihoods, 2);			% (K,D) reduces to (K) of y-P.L
	GRAD(:,1:Num.Vars) = -DATA.X .* repmat(YP,1,Num.Vars);				% (K,V) for Gradient of Locations
		    
	% LogNormal Adjustment for Gradient of Locations
	if ~isempty(Dist.LogNormal_Vars)
		% Build a reshaped Beta_Draws array with only log-normally distributed parameters
		BETA_Draws_KDV = permute(BETA_Draws_DVC(:,LogNormal_Spots,:),[3 4 1 2]);% Rearrange LogNormal Beta Draws
		for A = 2:Num.Alts																			
			BETA_Draws_KDV(:,A,:,:) = BETA_Draws_KDV(:,1,:,:);				% Fill in Alts with values
		end % A
		BETA_Draws_KDV = reshape(BETA_Draws_KDV,[Num.K_CaseAlt,Num.Draws(1),length(Dist.LogNormal_Vars)]);
		
		% Recalculate the log-normal parameters' gradient
		for L = 1:length(Dist.LogNormal_Vars)															% For each LogNormal parameter
			YP = sum(BETA_Draws_KDV(:,:,L) .* (DATA.Y_D - Probabilities) .* Likelihoods, 2);	% Redo YP
			GRAD(:,Dist.LogNormal_Vars(L)) = -DATA.X(:,Dist.LogNormal_Vars(L)) .* YP;						% Redo GRAD
		end % L
   end
			
	% Gradient of Spreads
	Num_Vars = Num.Vars;
	Num_Cases = Num.Cases;
	Num_Alts = Num.Alts;
	Num_Random_Vars = Num.Random_Vars;
	DATA_KR = DATA.X(:,Dist.Random_Vars);										% Discard the non-random variables to leave (K,R)
	MovingYP = (DATA.Y_D - Probabilities) .* Likelihoods;				% (K,D)
	for D = 1:Num.Draws(1)
	for C = 1:Num_Cases
	for A = 1:Num_Alts
	for R = 1:Num_Random_Vars
		K = C + (A-1)*Num_Cases;
		MovingData = Std_Draws_DVC(D,R,C) .* DATA_KR(K,R);
		GRAD(K,Num_Vars+R) = GRAD(K,Num_Vars+R)-( MovingData .* MovingYP(K,D) );
	end % D
	end
	end
	end
	    
	GRAD(isnan(GRAD)) = 0;
	    
	% Collapse GRAD to Cases then to Groups
	GRAD = reshape(GRAD, Num.Cases, Num.Alts, Num.Vars+Num.Random_Vars);	% (C,A,V) 
	GRAD = squeeze(sum(GRAD,2));	% Reduced to cases (C,V)
	if Module.Panel
		for G = 1:DATA.G.Num
			GRAD_G(:,G) = sum(GRAD(DATA.G.Begins(G):DATA.G.Ends(G),:),1)';	% add up across the group
		end % G
	else
		GRAD_G = GRAD';
	end % if
	
	% Insert zero lines into GRAD_G for non-random (unused) location parameters
	GRAD_G(Num.Vars+Dist.Random_Vars,:) = GRAD_G(Num.Vars+1:end,:);
	if size(GRAD_G,1) < length(Order_of_Meta_DATA)
		GRAD_G(length(Order_of_Meta_DATA),1) = 0;
	end
	
	GRAD_G = GRAD_G(Order_of_Meta_DATA,:);	% Reorder into alternating Loc/Spr from All Locs before All Sprs
	GRAD_G = Keys.Connector * GRAD_G;		% Collapse Gradients into Free parameters
	GRAD_G = GRAD_G(Keys.Free,:);			% Drop points that are not free
    Grad_MXL = sum(GRAD_G,2);				% Add up the gradient across all cases
	BHHH = GRAD_G * (GRAD_G');				% Add up the outer product of the gradient across all cases
	[G_BETA, G_THETA] = create_working_PAR (Grad_MXL', zeros(size(PAR_lock)), Keys.Unlock, Keys.Connector); % / Num.Cases;
	
	Hess_MXL = BHHH;
else
	Grad_MXL = zeros(length(PAR_open),1);
	Hess_MXL = zeros(length(PAR_open),length(PAR_open));
	BHHH = Hess_MXL;
end % Calc_Gradient


% Unscaling of BETA for Reporting
BETA_to_show = BETA./DATA.Max_Adj.A;
THETA_to_show = THETA./DATA.Max_Adj.A;

% Record Current_Parameters global variable for restartability
Current_Parameters.BETA = BETA_to_show;
Current_Parameters.THETA = THETA_to_show;		


% Results Output
if Prefs.VerboseOutput
	if Calc_Gradient												% Show results when Gradient is calculated
		Gradient_to_show1 = G_BETA./DATA.Max_Adj.A;
		Gradient_to_show2 = G_THETA./DATA.Max_Adj.A;
		relay ('Variable Name\tBETA-Loc\tBETA-Spr\tGRAD-Loc\tGRAD-Spr\n'); 
		for EachVar = 1:length(BETA_to_show)
			relay ('%-13s\t', Var_Names.BETA(EachVar,:));
			relay ('%8.8f\t', BETA_to_show(EachVar));
			relay ('%8.8f\t', THETA_to_show(EachVar));
			relay ('%8.8f\t', Gradient_to_show1(EachVar));
			relay ('%8.8f\n', Gradient_to_show2(EachVar));
		end % EachVar
	else														% Show results when Gradient is not calculated
		relay ('Variable Name\tBETA-Loc\tBETA-Spr\n'); 
		for EachVar = 1:length(BETA_to_show)
			relay ('%-13s\t', Var_Names.BETA(EachVar,:));
			relay ('%8.8f\t', BETA_to_show(EachVar));
			relay ('%8.8f\n', THETA_to_show(EachVar));
		end % EachVar
	end % Calc_Gradient
	relay ('LL: %1.8f\n',-LL);
%	timestamp;
	
	% Check for log file writing
	if LogCheck == ftell(LogFile)
		fprintf('RECORDING ERROR : LOG FILE WRITE BYTE UNCHANGED\n');
	else
		LogCheck = ftell(LogFile);
	end
end % if Prefs.VerboseOutput
%----------------------------------------------------------------------------------------------------------
function [PAR_open, PAR_lock] = create_lock_unlock_PAR (Free_PARs, BETA, THETA, MU)
% This function takes a vectors for BETA, THETA, and MU in functional order
% and makes two vectors of unconstrained and constrained parameters

if nargin == 4
	PAR = [BETA, THETA, MU];
elseif nargin == 3
	PAR = [BETA, THETA];
elseif nargin == 2
	PAR = [BETA];
end

PAR_open = PAR(  Free_PARs );
PAR_lock = PAR( ~Free_PARs );

if size(PAR_open,2) ~= 1 
	PAR_open = PAR_open';
end
if size(PAR_lock,2) ~= 1
	PAR_lock = PAR_lock';
end

%-----------------------------------------------------------------------
function [BETA, THETA, MU] = create_working_PAR (PAR_open, PAR_lock, Unlock_Key, Connector)
% This function takes two vectors of unconstrained and constrained parameters and makes
% three standard functional parameter vectors
% INPUT:
%	PAR_open	Row Vector		The unconstrained parameters
%	PAR_lock	Row Vector		The constrained parameters
%	Unlock_Key	Row Vector		A key with the locations of each vector parameter in the parts
%	Connections 3-d Array		An array containing inter-parameter contraints

global Num

if size(PAR_open,2) ~= 1 
	PAR_open = PAR_open';
end
if size(PAR_lock,2) ~= 1
	PAR_lock = PAR_lock';
end

PAR_combine = [PAR_open; PAR_lock];
PAR = PAR_combine(Unlock_Key)';
PAR = PAR * Connector;

BETA = PAR(1:Num.PAR.BETA);
THETA = PAR(Num.PAR.BETA+(1:Num.PAR.THETA));
MU = PAR(Num.PAR.BETA+Num.PAR.THETA+(1:Num.PAR.MU));

%-----------------------------------------------------------------------
function [Simple_Structure, Dense_Structure] = create_simple_structure (Structure_Vector)
% Simple_Structure has the parent of each nest noted
% Dense_Matrix has the full lineage of each nest

global Num

Structure_Length = length(Structure_Vector);	% define structure size
Simple_Structure = logical(zeros(Num.Nests,Num.Nests));		% initialize empty Simple_Structure
for Item = 1:Num.Nests-1
	Simple_Structure(Item, Structure_Vector(Item)) = 1;
end

Dense_Structure = eye(Num.Alts,Num.Nests);

for Item = Num.Alts+1:Num.Nests
	Nest_Children = Simple_Structure(:,Item);
	for B = find(Nest_Children)'
		Dense_Structure(:,Item) = Dense_Structure(:,Item) | Dense_Structure(:,B);
	end
end

%-----------------------------------------------------------------------
function [Draws] = draw_hypercube (Height, Width, Pages)
% creates a modified randomized latin hypercube draw of a IID multivariate uniform 
%Num.Draws(1), Num.Vars, Num.Items

Draws = zeros(Height, Width, Pages);	% Initialize holder

for EachPage = 1:Pages
	Spacing = 1/Height;
	Seed = rand(1,Width) .* Spacing;			%Creates a column vector of independant variables inside the smallest cell
	UniformGrid = zeros(Height, Width);
	UniformGrid(1,:)= Seed;
	
	for EachRow = 2:Height
		UniformGrid(EachRow,:) = UniformGrid(EachRow-1,:)+Spacing;
	end % EachRow
	
	for EachCol = 1:Width
		Draws(:,EachCol,EachPage) = UniformGrid(randperm(Height),EachCol);
	end % EachCol
end % EachPage

%-----------------------------------------------------------------------
function [Halton_Draws] = halton(Generator, Num_Draws, Num_Random_Vars, Num_Cases)

if nargin == 3
	Num_Cases = 1
end

if ~isa(Generator,'function_handle')
	Generator = str2func(Generator);
end

first100primes=[2, 3, 5, 7, 11, 13, 17, 19, 23, 29, ...
                31, 37, 41, 43, 47, 53, 59, 61, 67, 71, ...
                73, 79, 83, 89, 97, 101, 103, 107, 109, 113, ...
                127, 131, 137, 139, 149, 151, 157, 163, 167, 173, ...
                179, 181, 191, 193, 197, 199, 211, 223, 227, 229, ...
                233, 239, 241, 251, 257, 263, 269, 271, 277, 281, ...
                283, 293, 307, 311, 313, 317, 331, 337, 347, 349, ...
                353, 359, 367, 373, 379, 383, 389, 397, 401, 409, ...
                419, 421, 431, 433, 439, 443, 449, 457, 461, 463, ...
                467, 479, 487, 491, 499, 503, 509, 521, 523, 541];
relay('Creating %0.0f Draws for each of',Num_Draws);
relay(' %0.0f relevant cases. \n',Num_Cases);

Used_Primes = first100primes(1: Num_Random_Vars)
Num_Draws_To_Drop = 10;

D = Num_Draws_To_Drop + (Num_Draws * Num_Cases);

for EachVar = 1:Num_Random_Vars
	P = first100primes(EachVar);
	P_Draws = feval(Generator, D, P);
	Halton_Draws(:,EachVar,:) = reshape(P_Draws(Num_Draws_To_Drop+1:end),[Num_Draws 1 Num_Cases]);
end

%-----------------------------------------------------------------------
function [Halton_Seq] = halton_regular(D,P)
% This function creates a halton seq
% INPUTS:
%	D	Scalar	The number of random draws to create
%	P	Scalar	The [prime] number used to gerenate the sequence

Required_Length = D+1;
SEQ = 0:P-1;											% Create the regular sequence for P
Halton_Seq = SEQ;										% Reset the Halton_Seq series to starter set
MadeSoFar = length(Halton_Seq);							% Reset the MadeSoFar counter to start
Divisor = P;											% Reset Divider to P

while MadeSoFar < Required_Length						% While the Halton_Seq is too short
	Halton_Seq = Halton_Seq * P;							% Scale up the Seq for the next power of P
	Divisor = Divisor * P;									% Scale up Divisor for the next power of P
	Next = [];												% Clear the Replacement Seq
	for i = 1:P
		Next(end+1:end+MadeSoFar) = Halton_Seq + SEQ(i);	% Build the next segment of the seq
		if length(Next) > Required_Length, break, end		% If it's now long enough, stop
	end
	Halton_Seq = Next;										% Trade out the Seq for the better one
	MadeSoFar = length(Halton_Seq);							% Count how many MadeSoFar
end

Halton_Seq = Halton_Seq(2:Required_Length) / Divisor;		% divide out by the reached power of P

%-----------------------------------------------------------------------
function [Halton_Seq] = halton_shuffle(D,P)
% This function creates a shuffled halton
% INPUTS:
%	D	Scalar	The number of random draws to create
%	P	Scalar	The [prime] number used to gerenate the sequence

Halton_Seq = halton_regular(D,P);		% Create a regular Halton
Halton_Seq = Halton_Seq(randperm(D));	% Shuffle the sequence

%-----------------------------------------------------------------------
function [Halton_Seq] = halton_scramble(D,P)
% This function creates a scrambled halton
% INPUTS:
%	D	Scalar	The number of random draws to create
%	P	Scalar	The [prime] number used to gerenate the sequence

Required_Length = D+1;
SEQ = randperm(P)-1;									% Create the scrambled sequence for P
Halton_Seq = SEQ;										% Reset the Halton_Seq series to starter set
MadeSoFar = length(Halton_Seq);							% Reset the MadeSoFar counter to start
Divisor = P;											% Reset Divider to P

while MadeSoFar < Required_Length						% While the Halton_Seq is too short
	Halton_Seq = Halton_Seq * P;							% Scale up the Seq for the next power of P
	Divisor = Divisor * P;									% Scale up Divisor for the next power of P
	Next = [];												% Clear the Replacement Seq
	for i = 1:P
		Next(end+1:end+MadeSoFar) = Halton_Seq + SEQ(i);	% Build the next segment of the seq
		if length(Next) > Required_Length, break, end		% If it's now long enough, stop
	end
	Halton_Seq = Next;										% Trade out the Seq for the better one
	MadeSoFar = length(Halton_Seq);							% Count how many MadeSoFar
end

Halton_Seq = Halton_Seq(2:Required_Length) / Divisor;		% Finally, divide out by the reached power of P

%-----------------------------------------------------------------------
function x = trianglize ( Uniform, Peak, Spread )
% Transforms a uniform random value to a triangle random value

TopHalf = (Uniform > 0.5);
Bottom_Side = Peak - Spread + (Spread * sqrt(2 * Uniform));
Top_Side = Peak + Spread - (Spread * sqrt (2 * (1 - Uniform)));

x = (Bottom_Side .* ~TopHalf) + (Top_Side .* TopHalf);

%-----------------------------------------------------------------------
function [Draws] = gumbelize (Uniform, Location, VarianceFactor)
% creates a modified randomized latin hypercube draw of a IID mutlivariate Gumbel

% GumbelFromCDF = exp(-exp(-VarianceFactor*(Epsilon-Location)))
% CDFfromGumbel	= (-log(-log(Epsilon))*VarianceFactor)+Location  

if nargin == 1
	Location = 0;
	VarianceFactor = 1;
end

Draws = (-log(-log(Uniform))*VarianceFactor)+Location;

%-----------------------------------------------------------------------
function [] = relay(TextItem,varargin)
% RELAY (TextItem,VariableItem) : This function outputs to both screen and logfile
% 	Essentially, this command duplicates the work of two fprintf commands, and
% 	checks to see that the disk file is being written correctly.
% INPUTS:
%	TextItem		String	Text to print, including formats of variables as per fprintf command
%	varargin		Any		(Optional) The associated variable to print, as per fprintf command

global LogFile Module Prefs

if ~Prefs.SuppressAllOutput
Num_Bytes_Written = 0;
if nargin > 1	% For printing output that includes a variable
	try
		Num_Bytes_Written = fprintf (LogFile,TextItem, varargin{:});
	end
	if Num_Bytes_Written == 0
		fprintf('RECORDING ERROR : FAILED TO RECORD TO LOG FILE : ');
	end
	fprintf (TextItem, varargin{:});
else
	try
		Num_Bytes_Written = fprintf (LogFile,TextItem);
	end
	if Num_Bytes_Written == 0
		fprintf('RECORDING ERROR : FAILED TO RECORD TO LOG FILE : ');
	end
	fprintf (TextItem);
end

end % Prefs.SuppressAllOutput

%-----------------------------------------------------------------------
function [] = report(TextItem,varargin)
% REPORT (TextItem,VariableItem) : This function outputs to both screen and report file
% 	Essentially, this command duplicates the work of two fprintf commands, and
% 	checks to see that the disk file is being written correctly.
% INPUTS:
%	TextItem		String	Text to print, including formats of variables as per fprintf command
%	varargin		Any		(Optional) The associated variable to print, as per fprintf command

global ReportFile LogFile


if nargin > 1	% For printing output that includes a variable
	fprintf (ReportFile, TextItem, varargin{:});
	Num_Bytes_Written = fprintf (LogFile,TextItem, varargin{:});
	if Num_Bytes_Written == 0
		fprintf('RECORDING ERROR : FAILED TO RECORD TO LOG FILE : ');
	end
	fprintf (TextItem, varargin{:});	
else
	fprintf (ReportFile,TextItem);
	Num_Bytes_Written = fprintf (LogFile,TextItem);
	if Num_Bytes_Written == 0
		fprintf('RECORDING ERROR : FAILED TO RECORD TO LOG FILE : ');
	end
	fprintf (TextItem);
end

%-----------------------------------------------------------------------
function [] = write_report(filename, LL_Converge, LL_Zero, varnames, params, t_stats, std_errs, cov_matrix, converge_tol, num_iter)
% This will write the report.

global Prefs Version_Number Module

% Report of Results
report(['\n------------------------------------------------------------------------']);
report(['\nReport of Results: ' Prefs.ReportTitle '\n']);

report('Log Likelihood at Convergence:\t%0.3f\n',-LL_Converge);
report('Log Likelihood with Parameters at Zero:\t%0.3f\n',-LL_Zero);
report('Rho Squared (Unadjusted): %0.3f\n',(LL_Zero-LL_Converge)/LL_Zero);
report('\n');		

spacer = repmat(' ',[1 length(varnames(1,:))]);
spacer(1:9) = 'Variables';

Report_Format = ['%-9.' num2str(Prefs.Signif_Digits) 'g\t'];

report([spacer '\tParam    \tt-Stat v0\tStdError \tCovariance Matrix of the Estimate...\n']);
for V = 1:length(params)
	report('%-9s\t', varnames(V,:));
	report(Report_Format,params(V));
	report(Report_Format,t_stats(V));
	report(Report_Format,std_errs(V));
	report(Report_Format,cov_matrix(:,V));
	report('\n');		
end
report('\nConvergance Tolerance:\t%0.3g',converge_tol);		
report('\nNumber of Directional Search Iterations:\t%0.0f',num_iter);		
report('\nThis model was estimated using logit.m version %0.0f.\n',Version_Number);
fprintf('Report Filename: %s\n',['Report ' filename '.txt']);
report('\n');	
if num_iter < 5 && Module.BFGS
	report('WARNING: The BFGS approximation of the hessian matrix may be inaccurate due to the low number of iterations.\n');
	report('         Use caution in interpreting the standard error of the estiamte.\n');
end

%-----------------------------------------------------------------------
function [] = timestamp(CallType)
% Reports time since process began and time since last timestamp call
% When called without arguments, prints elapsed time since last call, and elapsed total time
% When called with argument, prints time of call labelled with the CallType

global Time_Begin Time_IterateBegin LogFile

if nargin == 1
	fprintf (LogFile,['\n' CallType ' : %s\n'],datestr(clock,'yyyy-mm-dd HH:MM:SS'));
	if all(CallType(1:8) == 'End Time')
		Total_Seconds = etime(clock,Time_Begin);
		Total_Minutes = fix(Total_Seconds/60);
		Total_Seconds = rem(Total_Seconds,60);
	%	fprintf (LogFile,'Total Elapsed Time:   %2.0f minutes', Total_Minutes);
	%	fprintf (LogFile,' %1.2f seconds\n', Total_Seconds);
		relay ('Total Elapsed Time:   %2.0f minutes', Total_Minutes);
		relay (' %1.2f seconds\n', Total_Seconds);
	end
    Time_IterateBegin = clock;
else
	Total_Seconds = etime(clock,Time_Begin);
	Process_Seconds = etime(clock,Time_IterateBegin);
	Total_Minutes = fix(Total_Seconds/60);
	Total_Seconds = rem(Total_Seconds,60);
	Process_Minutes = fix(Process_Seconds/60);
	Process_Seconds = rem(Process_Seconds,60);
%    fprintf (LogFile,'Total Elapsed Time:   %2.0f minutes', Total_Minutes);
%   fprintf (LogFile,' %1.2f seconds\n', Total_Seconds);
%  fprintf (LogFile,'Process Elapsed Time: %2.0f minutes', Process_Minutes);
% fprintf (LogFile,' %1.2f seconds\n', Process_Seconds);
	relay ('Total Elapsed Time:   %2.0f minutes', Total_Minutes);
	relay (' %1.2f seconds\n', Total_Seconds);
	relay ('Process Elapsed Time: %2.0f minutes', Process_Minutes);
	relay (' %1.2f seconds\n', Process_Seconds);
	Time_IterateBegin = clock;
end
%-----------------------------------------------------------------------
function [] = error_check(Must_Be_True, If_Not_True, varargin)
% INPUT:
%	Must_Be_True Logical	A logical check that should pass true
%	If_Not_True	 String		The response in event of a failure
%	varargin	 Scalar		Optional values to report in the error string

if ~Must_Be_True
	if nargin == 2								% To print a static string warning
		error(If_Not_True);
	elseif nargin > 2							% To print a dynamic string warning
		error(If_Not_True,  varargin{:});
	end
end
%-----------------------------------------------------------------------
function [] = warn_check(Must_Be_True, If_Not_True, varargin)
% INPUT:
%	Must_Be_True Logical	A logical check that should pass true
%	If_Not_True	 String		The response in event of a failure
%	varargin	 Scalar		Optional values to report in the error string

if ~Must_Be_True
	if nargin == 2								% To print a static string warning
		relay(If_Not_True);
	elseif nargin > 2							% To print a dynamic string warning
		relay(If_Not_True,  varargin{:});
	end
end
%-----------------------------------------------------------------------
function [Choleski] = negative_choleski(dimensions)
% This function calculates the implied Choleski decomposition of a 
% maximally negative covariant number of random variables.

if dimensions==1
	Choleski = 1;

elseif dimensions==2
	Choleski = [1 -1; 0 0];
	
elseif dimensions==3
	Choleski = [1	-1/2	-1/2;...
	0	1170/1351	-1170/1351;...
	0	0	0];

elseif dimensions==4
	Choleski = [1	-1/3	-1/3	-1/3;...
	0	1121/1189	-1121/2378	-1121/2378;...
	0	0	881/1079	-881/1079;...
	0	0	0	0];

elseif dimensions==5
	Choleski = [1	-1/4	-1/4	-1/4	-1/4;...
	0	1860/1921	-620/1921	-620/1921	-620/1921;...
	0	0	461/505	-461/1010	-461/1010;...
	0	0	0	570/721	-570/721;...
	0	0	0	0	0];

elseif dimensions==6
	Choleski = [1	-1/5	-1/5	-1/5	-1/5	-1/5;...
	0	4704/4801	-1176/4801	-1176/4801	-1176/4801	-1176/4801;...
	0	0	684/721	-228/721	-228/721	-228/721;...
	0	0	0	2584/2889	-1292/2889	-1292/2889;...
	0	0	0	0	1488/1921	-1488/1921;...
	0	0	0	0	0	0];
	
elseif dimensions==7
	Choleski = [1	-1/6	-1/6	-1/6	-1/6	-1/6	-1/6;...
	0	141/143	-141/715	-141/715	-141/715	-141/715	-141/715;...
	0	0	1624/1681	-406/1681	-406/1681	-406/1681	-406/1681;...
	0	0	0	869/929	-869/2787	-869/2787	-869/2787;...
	0	0	0	0	1785/2024	-1012/2295	-1012/2295;...
	0	0	0	0	0	1429/1871	-1429/1871;...
	0	0	0	0	0	0	0];

else
	Choleski = eye(dimensions-1);
	Choleski(Choleski==0) = -1 / (dimensions-1);
	Choleski = chol(Choleski);
	Choleski(:,end+1) = -sum(Choleski,2);
	Choleski(end+1,:) = 0;

end

%-------------------------------------------------
function [Values] = truncate(Values,Max_Val)
Values(Values > abs(Max_Val)) = abs(Max_Val);
Values(Values < -abs(Max_Val)) = -abs(Max_Val);

%-----------------------------------------------------------------------
